@@ -181,15 +181,19 @@ select yn in "ftp" "git" "manual"; do
     esac
 done
 
-cd pynx
+if [ -d pynx ]; then
+    echo "Installing PyNX..."
+    cd pynx
 
-if [ -z $3 ];
-    then echo "No tag given - using git pynx master head"
-else
-    git checkout tags/$3
+    if [ -z $3 ];
+        then echo "No tag given - using git pynx master head"
+    else
+        git checkout tags/$3
+    fi
+
+    python setup.py install
 fi
 
-python setup.py install
 
 
 echo
