@@ -12,6 +12,7 @@ import sys
 import warnings
 from copy import deepcopy
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift, fftfreq
 from scipy.signal import convolve
 from ..operator import Operator, OperatorException, has_attr_not_none
 from ..version import __version__
@@ -144,14 +145,14 @@ class CDI:
             ny, nx = self.iobs.shape
             x, y = np.arange(-nx // 2, nx // 2, dtype=np.float32), \
                    np.arange(-ny // 2, ny // 2, dtype=np.float32)[:, np.newaxis]
-            return np.fft.fftshift(x) * self.pixel_size_object, np.fft.fftshift(y) * self.pixel_size_object
+            return fftshift(x) * self.pixel_size_object, fftshift(y) * self.pixel_size_object
         else:
             nz, ny, nx = self.iobs.shape
             x, y, z = np.arange(-nx // 2, nx // 2, dtype=np.float32), \
                       np.arange(-ny // 2, ny // 2, dtype=np.float32)[:, np.newaxis], \
                       np.arange(-nz // 2, nz // 2, dtype=np.float32)[:, np.newaxis, np.newaxis]
-            return np.fft.fftshift(x) * self.pixel_size_object, np.fft.fftshift(y) * self.pixel_size_object, \
-                   np.fft.fftshift(z) * self.pixel_size_object
+            return fftshift(x) * self.pixel_size_object, fftshift(y) * self.pixel_size_object, \
+                   fftshift(z) * self.pixel_size_object
 
     def get_x_y(self):
         return self.get_x_y_z()
@@ -227,7 +228,7 @@ class CDI:
         """
         self._from_gpu()
         if shift:
-            return np.fft.fftshift(self._obj)
+            return fftshift(self._obj)
         else:
             return self._obj
 
@@ -243,7 +244,7 @@ class CDI:
         self._from_gpu()
 
         if shift:
-            return np.fft.fftshift(self._support)
+            return fftshift(self._support)
         else:
             return self._support
 
@@ -256,13 +257,13 @@ class CDI:
         :return: the 2D or 3D CDI numpy data array
         """
         if shift:
-            return np.fft.fftshift(self.iobs)
+            return fftshift(self.iobs)
         else:
             return self.iobs
 
     def set_obj(self, obj, shift=False):
         """
-        Set the object data array.
+        Set the object data array. Assumed to be in object (not Fourier) space
 
         :param obj: the 2D or 3D CDI numpy data array (complex64 numpy array)
         :param shift: if True, the data array will be fft-shifted so that the center of the stored data is
@@ -270,13 +271,14 @@ class CDI:
         :return: nothing
         """
         if shift:
-            self._obj = np.fft.fftshift(obj).astype(np.complex64)
+            self._obj = fftshift(obj).astype(np.complex64)
         else:
             self._obj = obj.astype(np.complex64)
         if self._timestamp_counter <= self._cl_timestamp_counter:
             self._timestamp_counter = self._cl_timestamp_counter + 1
         if self._timestamp_counter <= self._cu_timestamp_counter:
             self._timestamp_counter = self._cu_timestamp_counter + 1
+        self._is_in_object_space = True
 
     def set_support(self, support, shift=False):
         """
@@ -288,7 +290,7 @@ class CDI:
         :return: nothing
         """
         if shift:
-            self._support = np.fft.fftshift(support).astype(np.int8)
+            self._support = fftshift(support).astype(np.int8)
         else:
             self._support = support.astype(np.int8)
         if self._timestamp_counter <= self._cl_timestamp_counter:
@@ -307,7 +309,7 @@ class CDI:
         :return: nothing
         """
         if shift:
-            self.iobs = np.fft.fftshift(iobs).astype(np.float32)
+            self.iobs = fftshift(iobs).astype(np.float32)
         else:
             self.iobs = iobs.astype(np.float32)
         self.iobs_sum = self.iobs[self.iobs > 0].sum()
@@ -330,7 +332,7 @@ class CDI:
         if mask is None:
             return
         if shift:
-            mask = np.fft.fftshift(mask).astype(np.int8)
+            mask = fftshift(mask).astype(np.int8)
         iobs = self.get_iobs()
         iobs[mask > 0] = -1e31
         self.set_iobs(iobs)
@@ -363,7 +365,10 @@ class CDI:
                             if (dx ** 2 + dy ** 2 + dz ** 2) <= island_radius ** 2:
                                 m[iz + dz, iy + dy, ix + dx] = True
             else:
-                nb = int(self.iobs.size * ratio / (3.14 * island_radius ** 2))
+                if island_radius == 0:
+                    nb = int(self.iobs.size * ratio)
+                else:
+                    nb = int(self.iobs.size * ratio / (3.14 * island_radius ** 2))
                 ny, nx = self.iobs.shape
                 iy = np.random.randint(island_radius, ny - island_radius, nb)
                 ix = np.random.randint(island_radius, nx - island_radius, nb)
@@ -396,11 +401,11 @@ class CDI:
         # Exclude zone-center
         if m.ndim == 2:
             ny, nx = m.shape
-            y, x = np.meshgrid(np.fft.fftfreq(ny), np.fft.fftfreq(nx), indexing='ij')
+            y, x = np.meshgrid(fftfreq(ny), fftfreq(nx), indexing='ij')
             m[np.sqrt(x ** 2 + y ** 2) < exclude_zone_center] = False
         else:
             nz, ny, nx = m.shape
-            z, y, x = np.meshgrid(np.fft.fftfreq(nz), np.fft.fftfreq(ny), np.fft.fftfreq(nx), indexing='ij')
+            z, y, x = np.meshgrid(fftfreq(nz), fftfreq(ny), fftfreq(nx), indexing='ij')
             m[np.sqrt(x ** 2 + y ** 2 + z ** 2) < exclude_zone_center] = False
         m = m > 0
 
@@ -414,12 +419,16 @@ class CDI:
             print('Initialized free mask with %d pixels (%6.3f%%)' % (n, 100 * n / m.size))
         self.set_iobs(self.iobs)  # This will set the timestamp
 
-    def set_free_pixel_mask(self, m, verbose=False):
+    def set_free_pixel_mask(self, m, verbose=False, shift=False):
         """
         Set the free pixel mask. Assumes the free pixel mask is correct (excluding center, bad pixels)
         :param m: the boolean mask (1=masked, 0 otherwise)
+        :param shift: if True, the data array will be fft-shifted so that the center of the stored data is
+                      in the corner of the array. [default: the array is already shifted]
         :return:
         """
+        if shift:
+            m = fftshift(m)
         # Clear previous free mask
         tmp = np.logical_and(self.iobs > -1e30, self.iobs < 0)
         self.iobs[tmp] = -(self.iobs[tmp] + 1)
@@ -434,7 +443,6 @@ class CDI:
             print('Set free mask with %d pixels (%6.3f%%)' % (n, 100 * n / m.size))
         self.set_iobs(self.iobs)  # This will set the timestamp
 
-
     def get_free_pixel_mask(self):
         return np.logical_and(self.iobs > -1e30, self.iobs < 0).astype(np.bool)
 
@@ -678,6 +686,8 @@ class CDI:
         config.attrs['NX_class'] = 'NXcollection'
         if process_parameters is not None:
             for k, v in process_parameters.items():
+                if k == 'free_pixel_mask':
+                    k = 'free_pixel_mask_input'
                 if v is not None:
                     if type(v) is dict:
                         # This can happen if complex configuration is passed on
@@ -691,6 +701,17 @@ class CDI:
         if self.iobs is not None and 'iobs_shape' not in config:
             config.create_dataset('iobs_shape', data=self.iobs.shape)
 
+        # Save the free pixel mask so that it can be re-used
+        mask_free_pixel = self.get_free_pixel_mask()
+        if mask_free_pixel.sum():
+            config.create_dataset('free_pixel_mask', data=fftshift(mask_free_pixel.astype(np.bool)), chunks=True,
+                                  shuffle=True, compression="gzip")
+            config['free_pixel_mask'].attrs['note'] = "Mask of pixels used for free log-likelihood calculation"
+            if mask_free_pixel.ndim == 3:
+                config['free_pixel_mask'].attrs['interpretation'] = 'scaler'
+            else:
+                config['free_pixel_mask'].attrs['interpretation'] = 'image'
+
         results = process_1.create_group("results")
         results.attrs['NX_class'] = 'NXcollection'
         llk = self.get_llk(normalized=True)
@@ -772,7 +793,7 @@ class CDI:
             if 'dt' in kwargs:
                 dt = kwargs['dt']
             if verbose:
-                print("%4s #%3d LLK= %8.2f[%8.2f](p) %8.2f[%8.2f](g) %8.2f[%8.2f](e), nb photons=%e, "
+                print("%4s #%3d LLK= %8.2f[%8.3f](p) %8.3f[%8.3f](g) %8.3f[%8.3f](e), nb photons=%e, "
                       "support:nb=%6d (%6.3f%%) average=%10.2f max=%10.2f, dt/cycle=%6.4fs" % (
                           algo, self.cycle, llk[0], llk[3], llk[1], llk[4], llk[2], llk[5], self.nb_photons_calc,
                           self.nb_point_support, self.nb_point_support / self._obj.size * 100,
@@ -906,7 +927,7 @@ def save_cdi_data_cxi(filename, iobs, wavelength=None, detector_distance=None, p
         detector_1.create_dataset("y_pixel_size", data=pixel_size_detector)
         detector_1["y_pixel_size"].attrs['units'] = 'm'
     if iobs_is_fft_shifted:
-        detector_1.create_dataset("data", data=np.fft.fftshift(iobs), chunks=True, shuffle=True,
+        detector_1.create_dataset("data", data=fftshift(iobs), chunks=True, shuffle=True,
                                   compression="gzip")
     else:
         detector_1.create_dataset("data", data=iobs, chunks=True, shuffle=True,
@@ -915,7 +936,7 @@ def save_cdi_data_cxi(filename, iobs, wavelength=None, detector_distance=None, p
     if mask is not None:
         if mask.sum() != 0:
             if iobs_is_fft_shifted:
-                detector_1.create_dataset("mask", data=np.fft.fftshift(mask), chunks=True, shuffle=True,
+                detector_1.create_dataset("mask", data=fftshift(mask), chunks=True, shuffle=True,
                                           compression="gzip")
             else:
                 detector_1.create_dataset("mask", data=mask, chunks=True, shuffle=True, compression="gzip")
