@@ -13,10 +13,13 @@ import timeit
 import types
 import warnings
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift
 from skimage.restoration.deconvolution import richardson_lucy
+from scipy.fftpack import fftn, ifftn, fftshift
 from scipy.ndimage import gaussian_filter
 from scipy.ndimage.measurements import center_of_mass
-import matplotlib.pyplot as plt
+
+from pynx.utils.matplotlib import pyplot as plt
 
 from ..utils.plot_utils import complex2rgbalin, complex2rgbalog, insertColorwheel, cm_phase
 from ..utils.math import llk_euclidian, llk_gaussian, llk_poisson
@@ -219,17 +222,17 @@ class ImshowRGBA(OperatorCDI):
             d = cdi.get_obj(shift=True)
             x, y = cdi.get_x_y()
             if cdi._support is not None:
-                support = np.fft.fftshift(cdi.get_support())
+                support = fftshift(cdi.get_support())
         else:
             if self.i is not None:
                 i = self.i - len(cdi.get_obj()) // 2
             else:
                 v = np.sum(abs(cdi.get_obj()), axis=(1, 2))
-                i = int(round(center_of_mass(np.fft.fftshift(v))[0])) - len(cdi.get_obj()) // 2
-            d = np.fft.fftshift(cdi.get_obj()[i])
+                i = int(round(center_of_mass(fftshift(v))[0])) - len(cdi.get_obj()) // 2
+            d = fftshift(cdi.get_obj()[i])
             x, y, z = cdi.get_x_y()
             if cdi._support is not None:
-                support = np.fft.fftshift(cdi.get_support()[i])
+                support = fftshift(cdi.get_support()[i])
         if self.kwargs_complex2rgba is None:
             kw = {}
         else:
@@ -259,12 +262,12 @@ class ImshowRGBA(OperatorCDI):
         if support is not None:
             if support.sum():
                 ix = np.nonzero(support.sum(axis=0))[0].take([0, -1])
-                vx = np.fft.fftshift(x.flat)[ix]
+                vx = fftshift(x.flat)[ix]
                 dx = vx[1] - vx[0]
                 plt.xlim(((vx[0] - dx * 0.1) * s, (vx[1] + dx * 0.1) * s))
 
                 iy = np.nonzero(support.sum(axis=1))[0].take([0, -1])
-                vy = np.fft.fftshift(y.flat)[iy]
+                vy = fftshift(y.flat)[iy]
                 dy = vy[1] - vy[0]
                 plt.ylim(((vy[0] - dy * 0.1) * s, (vy[1] + dy * 0.1) * s))
         plt.title("Object")
@@ -306,7 +309,7 @@ class ShowCDI(OperatorCDI):
                 i = self.i - len(cdi.get_obj()) // 2
             else:
                 v = np.sum(cdi.get_support(), axis=(1, 2))
-                i = int(round(center_of_mass(np.fft.fftshift(v))[0])) - len(cdi.get_obj()) // 2
+                i = int(round(center_of_mass(fftshift(v))[0])) - len(cdi.get_obj()) // 2
             obj = cdi.get_obj()[i]
             iobs = cdi.get_iobs()[0].copy()
             icalc = self.get_icalc(cdi, 0)
@@ -327,40 +330,40 @@ class ShowCDI(OperatorCDI):
         plt.clf()
         plt.subplot(221)
         if support is not None:
-            tmp = np.fft.fftshift(support)
+            tmp = fftshift(support)
             # Scale so that the max is at the 99 percentile relative to the number of points inside the support
             percent = 100 * (1 - 0.01 * support.sum() / support.size)
             max99 = np.percentile(abs(obj), percent)
-            plt.imshow(np.fft.fftshift(abs(obj)), origin='lower', vmin=0, vmax=max99, cmap=plt.cm.get_cmap('gray'))
+            plt.imshow(fftshift(abs(obj)), origin='lower', vmin=0, vmax=max99, cmap=plt.cm.get_cmap('gray'))
             if tmp.sum():
                 plt.xlim(np.nonzero(tmp.sum(axis=0))[0].take([0, -1]) + np.array([-10, 10]))
                 plt.ylim(np.nonzero(tmp.sum(axis=1))[0].take([0, -1]) + np.array([-10, 10]))
             plt.colorbar()
         else:
-            plt.imshow(np.fft.fftshift(abs(obj)), origin='lower', cmap=plt.cm.get_cmap('gray'))
+            plt.imshow(fftshift(abs(obj)), origin='lower', cmap=plt.cm.get_cmap('gray'))
         plt.title('Object Amplitude')
 
         plt.subplot(222)
         if support is not None:
-            tmp = np.fft.fftshift(support)
-            p = np.ma.masked_array(np.fft.fftshift(np.angle(obj)), mask=(tmp == 0))
+            tmp = fftshift(support)
+            p = np.ma.masked_array(fftshift(np.angle(obj)), mask=(tmp == 0))
             plt.imshow(p, vmin=-np.pi, vmax=np.pi, origin='lower', cmap=cm_phase)
             if tmp.sum():
                 plt.xlim(np.nonzero(tmp.sum(axis=0))[0].take([0, -1]) + np.array([-10, 10]))
                 plt.ylim(np.nonzero(tmp.sum(axis=1))[0].take([0, -1]) + np.array([-10, 10]))
             plt.colorbar()
         else:
-            plt.imshow(np.fft.fftshift(np.angle(obj)), cmap=cm_phase)
+            plt.imshow(fftshift(np.angle(obj)), cmap=cm_phase)
         plt.title('Object Phase')
 
         mi, ma = np.log10(max(iobs.min(), 0.5)), np.log10(iobs.max())
         plt.subplot(223)
         if icalc is not None:
-            plt.imshow(np.log10(np.fft.fftshift(icalc)), vmin=mi, vmax=ma, origin='lower')
+            plt.imshow(np.log10(fftshift(icalc)), vmin=mi, vmax=ma, origin='lower')
         plt.title('Calculated intensity')
 
         plt.subplot(224)
-        plt.imshow(np.log10(np.fft.fftshift(iobs)), vmin=mi, vmax=ma, origin='lower')
+        plt.imshow(np.log10(fftshift(iobs)), vmin=mi, vmax=ma, origin='lower')
         plt.title('Observed intensity')
         try:
             plt.draw()
@@ -402,7 +405,7 @@ class AutoCorrelationSupport(CPUOperatorCDI):
     Operator to calculate an initial support from the auto-correlation function of the observed intensity.
     """
 
-    def __init__(self, threshold=0.2):
+    def __init__(self, threshold=0.2, verbose=False):
         """
         Operator initialization
         :param threshold: pixels above the autocorrelation maximimum multiplied by the threshold will be included
@@ -410,12 +413,16 @@ class AutoCorrelationSupport(CPUOperatorCDI):
         """
         super(AutoCorrelationSupport, self).__init__()
         self.threshold = threshold
+        self.verbose = verbose
 
     def op(self, cdi):
-        tmp = np.abs(np.fft.fftn(cdi.iobs.astype(np.complex64)))
+        tmp = np.abs(fftn(cdi.iobs.astype(np.complex64)))
         thres = tmp.max() * self.threshold
         cdi._support = (tmp > thres).astype(np.int8)
         cdi.nb_point_support = cdi._support.sum()
+        if self.verbose:
+            print('AutoCorrelation: %d pixels in support (%6.2f%%), threshold = %f' %
+                  (cdi.nb_point_support, cdi.nb_point_support * 100 / tmp.size, thres))
         return cdi
 
 
@@ -514,9 +521,9 @@ class FT(CPUOperatorCDI):
 
     def op(self, cdi):
         if self.scale is True:
-            cdi._obj = np.fft.fftn(cdi._obj, norm="ortho")
+            cdi._obj = fftn(cdi._obj, overwrite_x=True) / np.sqrt(cdi._obj.size)
         else:
-            cdi._obj = np.fft.fftn(cdi._obj, norm=None)
+            cdi._obj = fftn(cdi._obj, overwrite_x=True)
             if (self.scale is not False) and (self.scale is not None):
                 cdi = Scale(self.scale) * cdi
         cdi._is_in_object_space = False
@@ -542,9 +549,9 @@ class IFT(CPUOperatorCDI):
 
     def op(self, cdi):
         if self.scale is True:
-            cdi._obj = np.fft.ifftn(cdi._obj, norm="ortho")
+            cdi._obj = ifftn(cdi._obj, overwrite_x=True) * np.sqrt(cdi._obj.size)
         else:
-            cdi._obj = np.fft.ifftn(cdi._obj, norm=None)
+            cdi._obj = ifftn(cdi._obj, overwrite_x=True)
             if (self.scale is not False) and (self.scale is not None):
                 cdi = Scale(self.scale) * cdi
         cdi._is_in_object_space = True
@@ -1036,15 +1043,15 @@ class GPS(CPUOperatorCDI):
         # Gaussian smoothing arrays in object and Fourier space
         if cdi.iobs.ndim == 2:
             ny, nx = cdi.iobs.shape
-            qx = np.fft.fftfreq(nx).astype(np.float32)
-            qy = np.fft.fftfreq(ny).astype(np.float32)
+            qx = fftfreq(nx).astype(np.float32)
+            qy = fftfreq(ny).astype(np.float32)
             qy, qx = np.meshgrid(qy, qx, indexing='ij')
             qz = 0
         else:
             nz, ny, nx = cdi.iobs.shape
-            qx = np.fft.fftfreq(nx).astype(np.float32)
-            qy = np.fft.fftfreq(ny).astype(np.float32)
-            qz = np.fft.fftfreq(nz).astype(np.float32)
+            qx = fftfreq(nx).astype(np.float32)
+            qy = fftfreq(ny).astype(np.float32)
+            qz = fftfreq(nz).astype(np.float32)
             qz, qy, qx = np.meshgrid(qz, qy, qx, indexing='ij')
 
         g_o = np.exp(-2 * np.pi ** 2 * self.sigma_o ** 2 * (qx ** 2 + qy ** 2 + qz ** 2))
@@ -1155,7 +1162,7 @@ class ML(CPUOperatorCDI):
             cdi = IFT() * cdi
         for cycle in range(self.nb_cycle):
             obj_grad, obj_grad_last = obj_grad_last, obj_grad
-            psi = np.fft.fftn(cdi._obj, norm="ortho")
+            psi = fftn(cdi._obj) / np.sqrt(cdi._obj.size)
             icalc = abs(psi) ** 2
             iobs = cdi.iobs.copy()
             iobs[cdi.iobs < 0] = icalc[cdi.iobs < 0]
@@ -1164,7 +1171,7 @@ class ML(CPUOperatorCDI):
                 cdi = LLK() * cdi
 
             # This calculates the iFT of the conjugate of [(1 - iobs/icalc) * psi]
-            obj_grad = np.fft.ifftn(psi.conj() * (1 - iobs / icalc), norm="ortho")
+            obj_grad = ifftn(psi.conj() * (1 - iobs / icalc)) * np.sqrt(psi.size)
 
             if cdi.llk_support_reg_fac > 0:
                 obj_grad += cdi.llk_support_reg_fac * (1 - cdi._support) * cdi._obj
@@ -1181,7 +1188,7 @@ class ML(CPUOperatorCDI):
                 beta = np.float32(max(0, beta_n / max(1e-20, beta_d)))
 
             obj_dir = beta * obj_dir - obj_grad
-            dpsi = np.fft.fftn(obj_dir, norm="ortho")
+            dpsi = fftn(obj_dir) / np.sqrt(obj_dir.size)
 
             gamma_n = ((psi.conj() * dpsi).real * (iobs / icalc - 1)).sum()
             gamma_d = (abs(dpsi) ** 2 - iobs * (
@@ -1258,8 +1265,8 @@ class SupportUpdate(OperatorCDI):
                 smooth_width = a * np.exp(-i / nb * np.log(a / b))
             else:
                 smooth_width = b
-        tmpobj = np.fft.fftshift(np.abs(cdi.get_obj()).astype(np.float32))
-        tmpobj = np.fft.fftshift(gaussian_filter(tmpobj, smooth_width))
+        tmpobj = fftshift(np.abs(cdi.get_obj()).astype(np.float32))
+        tmpobj = fftshift(gaussian_filter(tmpobj, smooth_width))
         support = cdi.get_support()
 
         # Get total number of photons and maximum intensity in the support
@@ -1320,22 +1327,22 @@ class ScaleObj(OperatorCDI):
     def op(self, cdi):
         if self.method.lower() == 'f':
             # Scale the object to match Fourier amplitudes
-            tmpcalc = np.abs(np.fft.fftn(cdi._obj, norm=None)) * (cdi.iobs >= 0)
+            tmpcalc = np.abs(fftn(cdi._obj)) * (cdi.iobs >= 0)
             tmpobs = np.sqrt(np.abs(cdi.iobs))
             scale = (tmpcalc * tmpobs).sum() / (tmpcalc ** 2).sum()
         elif self.method.lower() == 'i':
             # Scale object to match Fourier intensities
-            tmpcalc = np.abs(np.fft.fftn(cdi._obj, norm=None)) ** 2 * (cdi.iobs >= 0)
+            tmpcalc = np.abs(fftn(cdi._obj)) ** 2 * (cdi.iobs >= 0)
             scale = np.sqrt((tmpcalc * cdi.iobs).sum() / (tmpcalc ** 2).sum())
         elif self.method.lower() == 'p':
             # Scale object to match Poisson statistics
-            tmpcalc = np.abs(np.fft.fftn(cdi._obj, norm=None)) ** 2 * (cdi.iobs >= 0)
+            tmpcalc = np.abs(fftn(cdi._obj)) ** 2 * (cdi.iobs >= 0)
             scale = np.sqrt((tmpcalc * cdi.iobs).sum() / (tmpcalc ** 2).sum())
         else:
             # Scale object to match weighted intensities
             # Weight: 1 for null intensities, zero for masked pixels
             w = (1 / (np.abs(cdi.iobs) + 1e-6) * (cdi.iobs > 1e-6) + (cdi.iobs <= 1e-6)) * (cdi.iobs >= 0)
-            tmpcalc = np.abs(np.fft.fftn(cdi._obj, norm=None)) ** 2
+            tmpcalc = np.abs(fftn(cdi._obj)) ** 2
             scale = np.sqrt((w * tmpcalc * cdi.iobs).sum() / (w * tmpcalc ** 2).sum())
         cdi._obj *= scale
         if self.verbose:
@@ -1416,7 +1423,7 @@ class DetwinSupport(CPUOperatorCDI):
         else:
             # Get current support
             cdi._support_tmp = cdi._support.copy()
-            tmp = np.fft.fftshift(cdi._support)
+            tmp = fftshift(cdi._support)
             # Use center of mass to cut near middle
             c = center_of_mass(tmp)
             if self.axis == 0:
@@ -1425,7 +1432,7 @@ class DetwinSupport(CPUOperatorCDI):
                 tmp[:, int(round(c[1])):] = 0
             else:
                 tmp[:, :, int(round(c[2])):] = 0
-            cdi._support = np.fft.fftshift(tmp)
+            cdi._support = fftshift(tmp)
         return cdi
 
 
@@ -1506,7 +1513,9 @@ class ObjConvolve(CPUOperatorCDI):
 class EstimatePSF(CPUOperatorCDI):
     """
     Estimate the Point Spread Function. Applies to a CDI object in reciprocal space.
-    The PSF is liklely due to partial coherence and thus related to the Mutual Coherence Function.
+    The PSF is usually due to partial coherence and thus related to the Mutual Coherence Function.
+
+    TODO: the CPU version of EstimatePSF is not finished or tested (unused)
     """
 
     def __init__(self, reg_fac=1e-2, nb_cycle=1, calc_llk=False):
@@ -1535,7 +1544,8 @@ class EstimatePSF(CPUOperatorCDI):
         n = cdi._k_psf_n  # Kernel size = 2*n+1
 
         icalc = abs(cdi.get_obj(shift=True)).astype(np.float32) ** 2
-        iobs = (cdi.get_iobs(shift=True) * (cdi.iobs < 0) + icalc * (cdi.iobs < 0)).astype(np.float32)
+        iobs = cdi.get_iobs(shift=True)
+        iobs = (iobs * (iobs >= 0) + icalc * (iobs < 0)).astype(np.float32)
 
         # Use only the center of the data to estimate the PSF - much faster !
         if cdi._obj.ndim == 2:
