@@ -10,6 +10,7 @@ import warnings
 from .cuda_device import has_cuda
 from .opencl_device import has_opencl
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift
 
 
 class ProcessingUnitException(Exception):
@@ -38,6 +39,7 @@ if 'PYNX_PU' in os.environ:
 
 
 class ProcessingUnit(object):
+
     def __init__(self):
         # Generic parameters
         self.pu_language = None  # 'cpu', 'opencl' or 'cuda' - None if un-initialised
@@ -135,10 +137,11 @@ class ProcessingUnit(object):
             fft_axis = None
         if test_fft:
             gpu_speed = opencl_device.available_gpu_speed(cl_platform=platform, fft_shape=self.benchmark_fft_size,
-                                                          axes=fft_axis, min_gpu_mem=None, verbose=verbose)
+                                                          axes=fft_axis, min_gpu_mem=None, verbose=verbose,
+                                                          gpu_name=gpu_name)
         else:
             gpu_speed = opencl_device.available_gpu_speed(cl_platform=platform, fft_shape=None, axes=None,
-                                                          min_gpu_mem=None, verbose=verbose)
+                                                          min_gpu_mem=None, verbose=verbose, gpu_name=gpu_name)
         if len(gpu_speed) == 0:
             return self.set_device(verbose=verbose)
         if gpu_name is not None:
@@ -310,6 +313,19 @@ class ProcessingUnit(object):
             s += " [name=%s]" % str(gpu_name)
         raise Exception(s)
 
+    @classmethod
+    def get_context(cls, device):
+        """
+        Static method to get a context, using the static device context dictionary to avoid creating new contexts,
+        which will use up the GPU memory.
+        This abstract function is implemented in CUProcessingUnit and CLProcessingUnit classes, the base version should
+        never be called.
+        :param device: the pyCUDA or pyOpenCL device for which a context is desired
+        :return: the OpenCL or
+        """
+        raise Exception('ProcessingUnit.get_context() is an abstract function, which should not be called. '
+                        'You probably want to use a CLProcessingUnit or CUProcessingUnit.')
+
     def finish(self):
         """
         Wait till all processing unit calculations are finished
@@ -341,11 +357,10 @@ class ProcessingUnit(object):
         :return: the result of the FFT transform
         """
         # TODO: derive and accelerate this function for different processing units
+        d = fftn(d, axes=axes)
         if normalize:
-            norm = "ortho"
-        else:
-            norm = None
-        return np.fft.fftn(d, axes=axes, norm=norm)
+            d /= np.sqrt(d.size)
+        return d
 
     def ifft(self, d, axes=None, normalize=False):
         """
@@ -359,11 +374,10 @@ class ProcessingUnit(object):
         :return: the result of the inverse FFT transform
         """
         # TODO: derive and accelerate this function for different processing units
+        d = fftn(d, axes=axes)
         if normalize:
-            norm = "ortho"
-        else:
-            norm = None
-        return np.fft.ifftn(d, axes=axes, norm=norm)
+            d *= np.sqrt(d.size)
+        return d
 
 
 # The default processing unit
