@@ -22,13 +22,10 @@ except ImportError:
     pass
 
 from . import ProcessingUnit
+from .cu_resources import cu_resources
 
 from pynx.utils.math import test_smaller_primes
 
-# Keep a list of devices used, and the associated dictionary of contexts to avoid creating new ones, with a
-# reference counting for safe deletion.
-# TODO: keep a more sophisticated list of contexts for different tasks (processing, copy in/out, etc...)
-cu_device_dict = {}
 
 class CUProcessingUnit(ProcessingUnit):
     """
@@ -46,16 +43,6 @@ class CUProcessingUnit(ProcessingUnit):
         self.cufft_batch = None  # cufft axes
         self.cu_arch = None
 
-    def __del__(self):
-        if self.cu_ctx is not None:
-            cu_device_dict[self.cu_device][self.cu_ctx] -= 1
-            if cu_device_dict[self.cu_device][self.cu_ctx] == 0:
-                # Maybe should be using sys.getrefcount() instead of manual reference couting ?
-                cu_device_dict[self.cu_device].pop(self.cu_ctx)
-                self.cu_ctx.pop()
-            self.cu_ctx = None
-
-
     def init_cuda(self, cu_ctx=None, cu_device=None, fft_size=(1, 1024, 1024), batch=True, gpu_name=None, test_fft=True,
                   verbose=True):
         """
@@ -77,16 +64,8 @@ class CUProcessingUnit(ProcessingUnit):
         assert test_smaller_primes(fft_size[-1], self.max_prime_fft, required_dividers=(2,)) \
                and test_smaller_primes(fft_size[-2], self.max_prime_fft, required_dividers=(2,))
 
-        if self.cu_device in cu_device_dict:
-            if len(cu_device_dict[self.cu_device]) > 0:
-                self.cu_ctx = list(cu_device_dict[self.cu_device].keys())[0]
-                cu_device_dict[self.cu_device][self.cu_ctx] += 1
-        else:
-            cu_device_dict[self.cu_device] = {}
-
         if self.cu_ctx is None:
-            self.cu_ctx = self.cu_device.make_context()
-            cu_device_dict[self.cu_device][self.cu_ctx] = 1
+            self.cu_ctx = CUProcessingUnit.get_context(self.cu_device)
 
         self.cu_options = ["-use_fast_math"]
         # TODO: KLUDGE. Add a workaround if the card (Pascal) is more recent than the driver...
@@ -95,6 +74,14 @@ class CUProcessingUnit(ProcessingUnit):
         #    self.cu_arch = 'sm_50'
         self.cu_init_kernels()
 
+    def get_context(device):
+        """
+        Static method to get a context, using the static device context dictionary to avoid creating new contexts,
+        which will use up the GPU memory.
+        :param device: the pyCUDA device for which a context is desired
+        """
+        return cu_resources.get_context(device)
+
     def cu_init_kernels(self):
         """
         Initialize kernels. Virtual function, must be derived.
