@@ -21,6 +21,8 @@ except:
     has_cuda = False
 import timeit
 
+from .cu_resources import cu_resources
+
 
 def cuda_device_fft_speed(d=None, fft_shape=(16, 256, 256), batch=True, verbose=False, nb_test=4, nb_cycle=1,
                           timing=False):
@@ -37,7 +39,7 @@ def cuda_device_fft_speed(d=None, fft_shape=(16, 256, 256), batch=True, verbose=
     :return: The computed speed in Gflop/s (if timing is False) or a tuple (flops, dt)
     """
     if d is not None:
-        ctx = d.make_context()
+        ctx = cu_resources.get_context(d)
 
     # TODO: support auto_allocate=False once skcuda version >=0.5.2 is out.
     if batch:
