@@ -5,6 +5,7 @@
 #       authors:
 #         Vincent Favre-Nicolin, favre@esrf.fr
 
+import gc
 import numpy as np
 
 try:
@@ -16,6 +17,8 @@ try:
 except:
     has_opencl = False
 
+from .cl_resources import cl_resources
+
 import timeit
 
 
@@ -24,10 +27,12 @@ def cl_device_fft_speed(d=None, fft_shape=(16, 256, 256), axes=(-1, -2), verbose
     """
     Compute the FFT calculation speed for a given OpenCL device.
     
-    :param d: the pyopencl.Device
-    :param fft_shape=(nz,ny,nx): the shape of the complex fft transform, treated as a stack of nz 2D transforms 
+    :param d: the pyopencl.Device. If not supplied, pyopencl.create_some_context() will be called, and a
+              device can be chosen interactively. This will result in a new context created for each call,
+              and is not efficient (the context memory cannot be freed).
+    :param fft_shape: (nz,ny,nx) the shape of the complex fft transform, treated as a stack of nz 2D transforms
                                  of size nx * ny, or as a single 3D FFT, depending on the value of 'axes'
-    :param axes=(1,2): the axes for the FFT. Default value is (-1,-2), which will perform a stacked 2d fft.
+    :param axes: (1,2) the axes for the FFT. Default value is (-1,-2), which will perform a stacked 2d fft.
                        Using None will perform a 3d fft.
     :param verbose: if True, print the speed and timing for the given transform
     :param nb_test: number of time the calculations will be repeated, the best result is returned
@@ -35,10 +40,12 @@ def cl_device_fft_speed(d=None, fft_shape=(16, 256, 256), axes=(-1, -2), verbose
     :param timing: if True, also return the time needed for a single FFT (dt)
     :return: The computed speed in Gflop/s (if timing is False) or a tuple (flops, dt)
     """
+    destroy_ctx = False
     if d is None:
         cl_ctx = cl.create_some_context()
+        destroy_ctx = True
     else:
-        cl_ctx = cl.Context([d])
+        cl_ctx = cl_resources.get_context(d)
     cl_queue = cl.CommandQueue(cl_ctx)
     cl_psi = cla.zeros(cl_queue, fft_shape, np.complex64)
 
@@ -63,6 +70,15 @@ def cl_device_fft_speed(d=None, fft_shape=(16, 256, 256), axes=(-1, -2), verbose
             dt = dtnew
         elif dtnew < dt:
             dt = dtnew
+
+    # Try to clean
+    del gpyfft_plan
+    cl_psi.data.release()
+    del cl_psi
+    if destroy_ctx:
+        del cl_ctx
+    gc.collect()
+
     nz, ny, nx = fft_shape
     if test_3d:
         # 3D FFT
