@@ -11,8 +11,9 @@ from __future__ import division
 import sys
 import time
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift
 from matplotlib.figure import Figure
-from matplotlib import pyplot as plt
+from pynx.utils.matplotlib import pyplot as plt
 from pynx import wavefront
 from pynx.wavefront import PropagateNearField
 from pynx.utils.plot_utils import complex2rgbalin, colorwheel
@@ -50,8 +51,7 @@ def probe_propagate(probe, propagation_range, pixel_size, wavelength, do_plot=Tr
     p = np.empty((nz, ny, nx), dtype=np.complex64)
     fwhm_v = np.empty(nz, dtype=np.float32)
     fwhm_h = np.empty(nz, dtype=np.float32)
-    w = wavefront.Wavefront(d=np.fft.fftshift(probe.astype(np.complex64)),
-                            wavelength=wavelength, pixel_size=pixel_size)
+    w = wavefront.Wavefront(d=fftshift(probe.astype(np.complex64)), wavelength=wavelength, pixel_size=pixel_size)
     i = 0
     px = pixel_size * 1e6 / 2
     vx = np.arange(-nx * px, nx * px, px * 2)  # in µm, we don't care about origin yet
