@@ -9,6 +9,7 @@
 import types
 import timeit
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift
 import pyopencl as cl
 import pyopencl.array as cla
 from pyopencl.elementwise import ElementwiseKernel as CL_ElK
@@ -647,7 +648,7 @@ class ShowPsi(CLOperatorBraggPtycho):
     def op(self, p):
         x, y, z = p.get_xyz(domain='probe')
         # We only show the first object mode
-        o = np.fft.fftshift(p._cl_psi.get()[0, 0, self.i])
+        o = fftshift(p._cl_psi.get()[0, 0, self.i])
         cpuop.show_3d(x, y, z, o, fig_num=self.fig_num, title=self.title, rotation=self.rotation)
         return p
 
