@@ -10,9 +10,10 @@ __all__ = ['ShowObj']
 import numpy as np
 from .braggptycho import OperatorBraggPtycho, BraggPtycho
 from ...utils.plot_utils import cm_phase, insertColorwheel
-import matplotlib.pyplot as plt
+from pynx.utils.matplotlib import pyplot as plt
 from scipy.ndimage.interpolation import rotate
 from scipy.interpolate import RegularGridInterpolator
+from ...utils.rotation import rotation_matrix
 
 
 #################################################################################################################
@@ -40,20 +41,8 @@ def show_3d(o, ortho_m, support=None, rotation=None, fig_num=-1, title=None, ext
     # Work in microns
     ortho_m = ortho_m * 1e6
     if rotation is not None:
-        # TODO: allow multiple axis
         ax, ang = rotation
-        c, s = np.cos(ang), np.sin(ang)
-        if ax == 'x':
-            # y, z = c * y - s * z, c * z + s * y
-            ortho_m = np.dot(np.array([[1, 0, 0], [0, c, -s], [0, s, c]], dtype=np.float32), ortho_m)
-        elif ax == 'y':
-            # z, x = c * z - s * x, c * x + s * z
-            ortho_m = np.dot(np.array([[c, 0, -s], [0, 1, 0], [s, 0, c]], dtype=np.float32), ortho_m)
-        elif ax == 'z':
-            # x, y = c * x - s * y, c * y + s * x
-            ortho_m = np.dot(np.array([[c, -s, 0], [s, c, 0], [0, 0, 1]], dtype=np.float32), ortho_m)
-        else:
-            raise Exception("show_3d(): unknown rotation axis '%s'" % ax)
+        ortho_m = np.dot(rotation_matrix(axis=ax, angle=ang), ortho_m)
 
     # Get coordinates in original array
     nz, ny, nx = o.shape
