@@ -8,44 +8,13 @@
 __all__ = ['Bragg2DPtychoData', 'Bragg2DPtycho', 'OperatorBragg2DPtycho']
 
 import numpy as np
-from scipy.interpolate import RegularGridInterpolator
 from ...operator import Operator, has_attr_not_none
+from ...utils.rotation import rotate, rotation_matrix
+from ...utils.history import History
 
 
 # TODO: Merge as much as possible with the 3D Bragg case (almost identical)
 
-def rotation_matrix(axis, angle):
-    """
-    Creates a numpy rotation matrix. The convention is the NeXus one so that a positive rotation of +pi/2:
-    - with axis='x', transforms +y into z
-    - with axis='y', transforms +z into x
-    - with axis='z', transforms +x into y
-    :param axis: the rotation axis, either 'x', 'y' or 'z'
-    :param angle: the rotation angle in radians
-    :return: the rotation matrix
-    """
-    c, s = np.cos(angle), np.sin(angle)
-    if axis.lower() == 'x':
-        return np.matrix([[1, 0, 0], [0, c, s], [0, -s, c]], dtype=np.float32)
-    elif axis.lower() == 'y':
-        return np.matrix([[c, 0, s], [0, 1, 0], [-s, 0, c]], dtype=np.float32)
-    else:
-        return np.matrix([[c, -s, 0], [s, c, 0], [0, 0, 1]], dtype=np.float32)
-
-
-def rotate(m, x, y, z):
-    """
-    Perform a rotation given a rotation matrix and x, y, z coordinates (which can be arrays)
-    :param m:
-    :param x:
-    :param y:
-    :param z:
-    :return: a tuple of x, y, z coordinated after rotation
-    """
-    return m[0, 0] * x + m[0, 1] * y + m[0, 2] * z, \
-           m[1, 0] * x + m[1, 1] * y + m[1, 2] * z, \
-           m[2, 0] * x + m[2, 1] * y + m[2, 2] * z
-
 
 class Bragg2DPtychoData(object):
     """Class for two-dimensional ptychographic data: observed diffraction and probe positions.
@@ -62,15 +31,16 @@ class Bragg2DPtychoData(object):
 
         :param iobs: 3d array with (nb_frame, ny,nx) shape observed intensity (assumed to follow Poisson statistics).
                      Data is assumed to be centered on the detector, and will be fft-shifted to be centered in (0,0).
-        :param positions: (z, y, x) tuple or 2d array with ptycho probe positions in meters. The coordinates
+        :param positions: (x, y, z) tuple or 2d array with ptycho probe positions in meters. The coordinates
                           must be in the laboratory reference frame given above.
-        :param mask: 2D mask (>0 means masked pixel) for the observed data. Can be None.
+        :param mask: 2D or 3D mask (>0 means masked pixel) for the observed data. Can be None. If 2D, it is assumed
+                     that the mask is the same for all frames.
         :param wavelength: wavelength of the experiment, in meters.
         :param detector= {rotation_axes:(('x', 0), ('y', pi/4)), 'pixel_size':55e-6, 'distance':1}:
                parameters for the detector as a dictionary. The rotation axes giving the detector orientation
                will be applied in order, i.e. to find the detector direction, a vector with x=y=0, z=1 is rotated
                by the axes in the given order.
-        :param scattering_vector=(hz, hy, hx): the coordinates of the central scattering vector for all frames, taking
+        :param scattering_vector=(sx, sy, sz): the coordinates of the central scattering vector for all frames, taking
                into account any tilt for multi-angle Bragg projection ptychography. The scattering vector coordinates
                units should be inverse meters (without 2pi multiplication), i.e. the norm should be 2*sin(theta)/lambda,
                where theta is the Bragg angle. Note that only the difference between the scattering vector and the
@@ -82,7 +52,7 @@ class Bragg2DPtychoData(object):
         # would require including detector crop coordinates and rotation axes...
 
         if scattering_vector is not None:
-            hz, hy, hx = scattering_vector
+            hx, hy, hz = scattering_vector
             hx0 = hx.mean()
             hy0 = hy.mean()
             hz0 = hz.mean()
@@ -101,13 +71,15 @@ class Bragg2DPtychoData(object):
             self.ds = tmp.copy(), tmp.copy(), tmp.copy()
 
         if iobs is not None:
+            assert iobs.shape[-2] == iobs.shape[-1]
             self.iobs = np.fft.fftshift(iobs, axes=(-2, -1)).astype(np.float32)
             # Total nb of photons is used for regularization
             self.iobs_sum = self.iobs.sum()
             if mask is not None:
-                mask3D = np.repeat(mask[np.newaxis, :, :], iobs.shape[0],
-                                   axis=0)  # tile mask to be a 3D array with (nb_frame, ny, nx) shape
-                self.mask = np.fft.fftshift(mask3D.astype(np.int8), axes=(-2, -1))
+                if mask.ndim ==2:
+                    mask = np.repeat(mask[np.newaxis, :, :], iobs.shape[0],
+                                       axis=0)  # tile mask to be a 3D array with (nb_frame, ny, nx) shape
+                self.mask = np.fft.fftshift(mask.astype(np.int8), axes=(-2, -1))
                 self.iobs[self.mask > 0] = -100
             else:
                 self.mask = None
@@ -115,19 +87,42 @@ class Bragg2DPtychoData(object):
             self.iobs = None
 
         self.wavelength = wavelength
-        self.posz, self.posy, self.posx = positions
+        self.posx, self.posy, self.posz = positions
         self.posx -= self.posx.mean()
         self.posy -= self.posy.mean()
         self.posz -= self.posz.mean()
         self.detector = detector
-        self.m = np.matrix([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float32)
+        self.m = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float32)
         for ax, angle in detector['rotation_axes']:
-            self.m *= rotation_matrix(ax, angle)
+            self.m = np.dot(rotation_matrix(ax, angle), self.m)
         self.im = np.linalg.inv(self.m)
 
         # Calculate ds coordinate in detector reference frame
         self.ds1 = rotate(self.im, self.ds[0], self.ds[1], self.ds[2])
 
+    def calc_ds(self, dix, diy):
+        """
+        Compute the shift of the scattering vector corresponding to a shift in pixels on the detector.
+        :param dix: shift in pixels on the detector along X (detector reference frame, left to right). Can be a vector.
+        :param diy: shift in pixels on the detector along Y (detector reference frame, top to bottom). Can be a vector.
+        :return: the scattering vector coordinates (sx, sy, sz) in the laboratory reference frame
+        """
+        sx0, sy0, sz0 = self.s0  # Mean scattering vector for all frames, in laboratory referennce frame
+        # Try to shift by 50 pixels
+        sxd = sx0 * self.detector['distance'] * self.wavelength
+        syd = sy0 * self.detector['distance'] * self.wavelength
+        szd = (sz0 + 1 / self.wavelength) * self.detector['distance'] * self.wavelength
+        # Change in scattering vector on detector
+        sxd1 = sxd + self.detector['pixel_size'] * dix
+        syd1 = syd + self.detector['pixel_size'] * diy
+        # New scattering vector on Ewald's sphere
+        print(sxd1, syd1, szd)
+        sd1n = np.sqrt(sxd1 ** 2 + syd1 ** 2 + szd ** 2)
+        sx1 = sxd1 / sd1n / self.wavelength
+        sy1 = syd / sd1n / self.wavelength
+        sz1 = (szd / sd1n - 1) / self.wavelength
+        return sx1, sy1, sz1
+
 
 class Bragg2DPtycho(object):
     """ Class for 2D Bragg ptychography data: object, probe, and observed diffraction.
@@ -139,12 +134,20 @@ class Bragg2DPtycho(object):
         Constructor.
         :param probe: the starting estimate of the probe, as a pynx wavefront object - can be 3D if modes are used.
         :param data: the Bragg2DPtychoData object with all observed frames, ptycho positions
-        :param support: the support of the object (1 inside, 0 outside) the object will be constrained to
+        :param support: the support of the object, with values ranging from 0 (outside) to 100 (inside). Intermediate
+                        values are used to smooth the object while keeping the array to int8 type. If the array type
+                        is bool, True values will be converted to 100.
         """
         self._probe2d = probe  # The original 2D probe as a Wavefront object
-        self._probe = None  # This will hold the probe as projected onto the 3D object
+        if self._probe2d is not None:
+            if self._probe2d.get().ndim == 2:
+                ny, nx = self._probe2d.get().shape
+                self._probe2d.set(self._probe2d.get().reshape(1, ny, nx))
         self._obj = None
         self.support = support
+        if self.support is not None:
+            if self.support.dtype is np.bool:
+                self.support = self.support.astype(np.int8) * 100
         self.data = data
         self._background = background
 
@@ -154,10 +157,10 @@ class Bragg2DPtycho(object):
         # Inverse of self.m
         self.im = None
 
-        # Voxel size in object and probe space
-        self.px1 = None
-        self.py1 = None
-        self.pz1 = None
+        # Voxel size of object array, in detector reference frame
+        self.pxo = None
+        self.pyo = None
+        self.pzo = None
 
         # Stored variables
         # self.scan_area_obj = None
@@ -178,6 +181,13 @@ class Bragg2DPtycho(object):
         self._cu_timestamp_counter = 0
         self.prepare()
 
+        # Record the number of cycles (RAAR, HIO, ER, CF, etc...), which can be used to make some parameters
+        # evolve, e.g. for support update
+        self.cycle = 0
+
+        # History record
+        self.history = History()
+
     def from_pu(self):
         """
         Get all relevant arrays from processing unit, if necessary
@@ -188,7 +198,7 @@ class Bragg2DPtycho(object):
                 if has_attr_not_none(self, '_cl_obj'):
                     self._obj = self._cl_obj.get()
                 if has_attr_not_none(self, '_cl_probe'):
-                    self._probe = self._cl_probe.get()
+                    self._probe2d.set(self._cl_probe.get(), shift=True)
                 if self._background is not None:
                     if has_attr_not_none(self, '_cl_background'):
                         self._background = self._cl_background.get()
@@ -196,7 +206,7 @@ class Bragg2DPtycho(object):
                 if has_attr_not_none(self, '_cu_obj'):
                     self._obj = self._cu_obj.get()
                 if has_attr_not_none(self, '_cu_probe'):
-                    self._probe = self._cu_probe.get()
+                    self._probe2d.set(self._cu_probe.get(), shift=True)
                 if self._background is not None:
                     if has_attr_not_none(self, '_cu_background'):
                         self._background = self._cu_background.get()
@@ -207,20 +217,32 @@ class Bragg2DPtycho(object):
         Get the probe data array. This will automatically get the latest data, either from GPU or from the host
         memory, depending where the last changes were made.
 
-        :param shift: if True, the data array will be fft-shifted so that the center of the data is in the center
-                      of the array, rather than in the corner (the default).
         :return: the 3D numpy data array (nb object modes, nyo, nxo)
         """
         self.from_pu()
-        return self._probe
+        return self._probe2d.get(shift=True)
+
+    def set_probe(self, pr):
+        """
+        Set the probe data array. This will automatically get the latest data, either from GPU or from the host
+        memory, depending where the last changes were made.
+
+        :param pr: the probe array, either 2D or 3D (modes) - will be converted to 3D if necessary. The array is
+                   assumed to be centered, and will be fft-shifted so that the probe is centered on the corners.
+        """
+        self.from_pu()
+        if pr.ndim == 2:
+            ny, nx = pr.shape
+            pr = pr.reshape(1, ny, nx)
+        self._probe2d.set(pr, shift=True)
+        self._timestamp_counter += 1
+        self._cpu_timestamp_counter = self._timestamp_counter
 
     def get_obj(self):
         """
         Get the object data array. This will automatically get the latest data, either from GPU or from the host
         memory, depending where the last changes were made.
 
-        :param shift: if True, the data array will be fft-shifted so that the center of the data is in the center
-                      of the array, rather than in the corner (the default).
         :return: the 3D numpy data array (nb object modes, nyo, nxo)
         """
         self.from_pu()
@@ -258,6 +280,9 @@ class Bragg2DPtycho(object):
         """
         if sup is not None:
             self.support = sup.astype(np.int8)
+            if sup.dtype is np.bool:
+                self.support *= 100
+
             self.support_sum = self.support.sum()
             self._timestamp_counter += 1
             self._cpu_timestamp_counter = self._timestamp_counter
@@ -305,34 +330,65 @@ class Bragg2DPtycho(object):
         Calculate projection parameters
         :return:
         """
-        self.calc_orthonormalisation_matrix()
-        self.init_probe()
+        self.prepare_orthonormalisation_matrix()
         self.init_obj()
         self.set_support(self.support)
 
-    def calc_orthonormalisation_matrix(self):
+    def get_orthonormalisation_matrix(self, rotation_axes=None):
         """
-        Calculate the orthonormalisation matrix to convert probe/object array coordinates (pixel coordinates in the
-        detector reference frame) to/from orthonormal ones in the laboratory reference frame.
-        This also initialises the voxel sizes in object and probe space
-        :return:
+        Get the orthonormalisation matrix which can transforms the object pixel coordinates to meters in the laboratory
+        reference frame. Optionally, add a few rotations to the transformation.
+        The orthonormal (laboratory) reference frame is following the NeXus/McStas convention:
+        - centered on the object center
+        - z is along the direct beam propagation
+        - y is vertical, upward,
+        - x is horizontal so that the frame is direct, i.e. is >0 towards the left as seen from the source.
+
+        The detector reference frame in object space, also used for the 3D probe and Psi back-propagation is such that:
+        - Z is going from the object to the detector
+        - X is parallel to the detector horizontal side, X>0 going right as seen from the sample
+        - Y is vertical, downward, when the detector is at the origin
+        - this corresponds to a detector origin which is 'top, left' as seen from the sample. The object array
+        origin is thus 'top, left, source-side' as seen from the X-ray source.
+
+        :param rotation_axes: a tuple of rotation axes, e.g. (('x', 0), ('y', pi/4)), describing the orientation of
+                              the detector. The different rotations are applied in order (left-to-right) to transform
+                              the pixel coordinates from a detector at origin (in the direct beam), to the coordinates
+                              in the orthonormal laboratory reference frame.
+        :return: a tuple of (orthonormalisation matrix, pxo, pyo, pzo), where pxo, pyo, pzo are the voxel dimensions in
+                 meters
         """
         npos, ny, nx = self.data.iobs.shape
         # Pixel size. Assumes square frames from detector.
         d = self.data.detector['distance']
         lambdaz = self.data.wavelength * d
         p = self.data.detector['pixel_size']
-        py, px = lambdaz / (p * ny), lambdaz / (p * nx)
+        pyo, pxo = lambdaz / (p * ny), lambdaz / (p * nx)
         # TODO: Find method to estimate optimal pixel size along z. Would depend on probe shape and multiple angles..
-        pz = max(px, py)
-        self.m = self.data.m.copy()
-        self.m[:, 0] *= -px  # - sign because image pixels origin is 'top, left' seen from sample
-        self.m[:, 1] *= -py  # - sign because image pixels origin is 'top, left' seen from sample
-        self.m[:, 2] *= pz
+        pzo = max(pxo, pyo)
+        m = self.data.m.copy()
+        if rotation_axes is not None:
+            for ax, angle in rotation_axes:
+                m = np.dot(rotation_matrix(ax, angle), m)
+
+        # - signs following NeXus convention for axes direction for detector and laboratory reference frames
+        m[:, 0] *= -pxo
+        m[:, 1] *= -pyo
+        m[:, 2] *= pzo
+        return m, pxo, pyo, pzo
+
+    def prepare_orthonormalisation_matrix(self):
+        """
+        Calculate the orthonormalisation matrix to convert probe/object array coordinates (pixel coordinates in the
+        detector reference frame) to/from orthonormal ones in the laboratory reference frame.
+        This also initialises the voxel sizes in object and probe space
+        :return: the orthonormalisation matrix
+        """
+        self.m, pxo, pyo, pzo = self.get_orthonormalisation_matrix()
         self.im = np.linalg.inv(self.m)
-        self.px1 = px
-        self.py1 = py
-        self.pz1 = pz
+        self.pxo = pxo
+        self.pyo = pyo
+        self.pzo = pzo
 
     def init_obj(self):
         """
@@ -454,7 +510,7 @@ class Bragg2DPtycho(object):
         :param multiple: the shape must be a multiple of that number. >=2
         :return: the 3D object shape (nzo, nyo, nxo)
         """
-        probe_shape = self._probe.shape[1:]
+        probe_shape = self.calc_probe_shape()
         ix, iy, iz = self.xyz_to_obj(self.data.posx, self.data.posy, self.data.posz)
 
         nz = int(2 * (abs(np.ceil(iz)) + 1).max() + probe_shape[0])
@@ -505,15 +561,13 @@ class Bragg2DPtycho(object):
     def get_xyz(self, rotation=None, domain='object'):
         """
         Get x,y,z orthonormal coordinates corresponding to the object grid.
-        :param domain='probe': the domain over which the xyz coordinates should be returned. It should either
+        :param domain: the domain over which the xyz coordinates should be returned. It should either
                                be 'object' (the default) or the probe, the only difference being that the object
-                               is extended to cover all the volume scanned by the shifted probe. The probe
-                               has the same size as the observed 3D data.
-        :param rotation=('z',np.deg2rad(-20)): optionally, the coordinates can be obtained after a rotation of the
-                                               object. This is useful if the object or support is to be defined as a
-                                               parallelepiped, before being rotated to be in diffraction condition.
-                                               The rotation can be given as a tuple of a rotation axis name (x, y or z)
-                                               and a counter-clockwise rotation angle in radians.
+                               is extended to cover all the volume scanned by the shifted probe.
+        :param rotation: ('z',np.deg2rad(-20)): optionally, the coordinates can be obtained after a rotation of the
+                         object. This is useful if the object or support is to be defined as a parallelepiped, before
+                         being rotated to be in diffraction condition. The rotation can be given as a tuple of a
+                         rotation axis name (x, y or z) and a counter-clockwise rotation angle in radians.
         :return: a tuple of (x,y,z) coordinates, each a 3D array
         """
         if domain == "probe":
@@ -560,45 +614,62 @@ class Bragg2DPtycho(object):
         pz = np.sqrt(tmp[0] ** 2 + tmp[1] ** 2 + tmp[2] ** 2)
         return pz, py, px
 
-    def init_probe(self):
+    def get_llk(self, noise=None, normalized=True):
         """
-        Calculate the probe over the object volume, given the 2D probe, the data and the object coordinates,
-        assuming that the probe is invariant along z.
-        :return: Nothing. Creates self._probe
+        Get the normalized log-likelihoods, which should converge to 1 for a statistically ideal fit.
+
+        :param noise: either 'gaussian', 'poisson' or 'euclidian', will return the corresponding log-likelihood.
+        :param normalized: if True, will return normalized values so that the llk from a statistically ideal model
+                           should converge to 1
+        :return: the log-likelihood, or if noise=None, a tuple of the three (poisson, gaussian, euclidian)
+                 log-likelihoods.
         """
-        # TODO: move this to an operator and kernel for faster conversion ?
-        print("Calculating probe on object grid")
-        x, y, z = self.get_xyz(domain='probe')
-        nz, ny, nx = x.shape
-        z0, z1 = z.min(), z.max()
+        n = 1
+        if normalized:
+            n = 1 / self.nb_obs
 
-        # Create 3D probe array in the laboratory frame
-        dz = z1 - z0
-        pr2d = self._probe2d.get(shift=True)
-        # TODO: take into account all probe modes
-        if pr2d.ndim == 3:
-            pr2d = pr2d[0]
-        nyp, nxp = pr2d.shape
-        if np.isclose(self.data.wavelength, self._probe2d.wavelength) is False:
-            raise Exception('BraggPtycho: probe and data wavelength are different !')
-        pixel_size_probe = self._probe2d.pixel_size
-        pr = np.empty((nz, nyp, nxp), dtype=np.complex64)
-        pr[:] = pr2d
+        if noise is None:
+            return self.llk_poisson * n, self.llk_gaussian * n, self.llk_euclidian * n
+        elif 'poiss' in str(noise).lower():
+            return self.llk_poisson * n
+        elif 'gauss' in str(noise).lower():
+            return self.llk_gaussian * n
+        elif 'eucl' in str(noise).lower():
+            return self.llk_euclidian * n
 
-        # Original probe coordinates
-        zp, yp, xp = np.arange(nz), np.arange(nyp), np.arange(nxp)
-        zp = (zp - zp.mean()) * (dz / nz)
-        yp = (yp - yp.mean()) * pixel_size_probe
-        xp = (xp - xp.mean()) * pixel_size_probe
-
-        # Interpolate probe to object grid
-        rgi = RegularGridInterpolator((zp, yp, xp), pr, method='linear', bounds_error=False, fill_value=0)
-        self._probe = rgi(np.concatenate((z.reshape(1, z.size), y.reshape(1, y.size),
-                                          x.reshape(1, x.size))).transpose()).reshape((1, nz, ny, nx)).astype(
-            np.complex64)
-        # To check view of object:
-        # pcolormesh(z[:,:,100],y[:,:,100],abs(p._probe3d[:,:,100]))
-        # np.savez_compressed('probe3d.npz', probe3dobj=self._probe, probe3d=pr, probe2d=pr2d, x=x, y=y, z=z)
+    def update_history(self, mode='llk', verbose=False, **kwargs):
+        """
+        Update the history record.
+        :param mode: either 'llk' (will record new log-likelihood, nb_photons_calc, average value..),
+        or 'algorithm' (will only update the algorithm).
+        :param verbose: if True, print some info about current process
+        :param kwargs: other parameters to be recorded, e.g. support_threshold=xx, dt=
+        :return: nothing
+        """
+        if mode == 'llk':
+            llk = self.get_llk()
+            algo = ''
+            update_object = True
+            update_probe = False
+            dt = 0
+            if 'algorithm' in kwargs:
+                algo = kwargs['algorithm']
+            if 'update_object' in kwargs:
+                update_object = kwargs['update_object']
+            if 'update_probe' in kwargs:
+                update_probe = kwargs['update_probe']
+            algo = algo_string(algo, self, update_object, update_probe)
+            if 'dt' in kwargs:
+                dt = kwargs['dt']
+            if verbose:
+                print("%-8s #%3d LLK= %8.2f(p) %8.2f(g) %8.2f(e), nb photons=%e, dt/cycle=%6.4fs" % (
+                          algo, self.cycle, llk[0], llk[1], llk[2], self.nb_photons_calc, dt))
+            self.history.insert(self.cycle, llk_poisson=llk[0], llk_gaussian=llk[1],
+                                llk_euclidian=llk[2], nb_photons_calc=self.nb_photons_calc,
+                                nb_obj=len(self._obj), nb_probe=len(self._probe2d._d), **kwargs)
+        elif 'algo' in mode:
+            if 'algorithm' in kwargs:
+                self.history.insert(self.cycle, algorithm=kwargs['algorithm'])
 
 
 class OperatorBragg2DPtycho(Operator):
@@ -611,3 +682,35 @@ class OperatorBragg2DPtycho(Operator):
         # display operators.
         p._timestamp_counter += 1
         p._cpu_timestamp_counter = p._timestamp_counter
+
+
+def algo_string(algo_base, p: Bragg2DPtycho, update_object, update_probe, update_background=False, update_pos=False):
+    """
+    Get a short string for the algorithm being run, e.g. 'DM/o/3p' for difference map with 1 object and 3 probe modes.
+
+    :param algo_base: 'AP' or 'ML' or 'DM'
+    :param p: the ptycho object
+    :param update_object: True if updating the object
+    :param update_probe: True if updating the probe
+    :param update_background: True if updating the background
+    :param update_pos: True if updating the positions
+    :return: a short string for the algorithm
+    """
+    s = algo_base
+
+    if update_object:
+        s += "/"
+        if len(p._obj) > 1:
+            s += "%d" % (len(p._obj))
+        s += "o"
+
+    if update_probe:
+        s += "/"
+        if len(p._probe2d._d) > 1:
+            s += "%d" % (len(p._probe))
+        s += "p"
+
+    if update_background:
+        s += "/b"
+
+    return s
