@@ -12,10 +12,14 @@ import time
 from sys import stdout
 import warnings
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift, fftfreq
+from scipy.spatial import cKDTree
+from scipy.interpolate import griddata
 from matplotlib.backends.backend_agg import FigureCanvasAgg
-from matplotlib import pyplot as plt
+from matplotlib.figure import Figure
+from pynx.utils.matplotlib import pyplot as plt
 from .ptycho import Ptycho, OperatorPtycho, algo_string
-from ..utils.plot_utils import show_obj_probe
+from ..utils.plot_utils import show_obj_probe, complex2rgbalin, colorwheel
 from .shape import get_view_coord
 from ..operator import has_attr_not_none, OperatorSum, OperatorPower, OperatorException
 from ..utils.math import ortho_modes
@@ -123,10 +127,10 @@ class CPUOperatorPtycho(OperatorPtycho):
                 need_calc_q = False
 
         if need_calc_q:
-            x = np.fft.fftfreq(nx) * nx
-            y = np.fft.fftfreq(ny)[:, np.newaxis] * ny
+            x = fftfreq(nx) * nx
+            y = fftfreq(ny)[:, np.newaxis] * ny
 
-            q = np.fft.fftshift(np.exp(1j * abs(factor) * (x * x + y * y)))
+            q = fftshift(np.exp(1j * abs(factor) * (x * x + y * y)))
             p._cpu_tmp_quad_phase = (q.astype(np.complex64), abs(factor))
 
     def timestamp_increment(self, p):
@@ -381,7 +385,7 @@ class ObjProbe2Psi(CPUOperatorPtycho):
                     op = p._probe[iprobe] * p._obj[iobj, cy:cy + ny, cx:cx + nx]
                     if p.data.near_field is False:
                         op *= p._cpu_tmp_quad_phase[0]
-                    p._psi[iobj, iprobe, i] = np.fft.fftshift(op)
+                    p._psi[iobj, iprobe, i] = fftshift(op)
         return p
 
 
@@ -400,9 +404,9 @@ class FT(CPUOperatorPtycho):
 
     def op(self, pty):
         if self.scale:
-            pty._psi = np.fft.fftn(pty._psi, axes=(-2, -1), norm="ortho")
+            pty._psi = fftn(pty._psi, axes=(-2, -1)) / np.sqrt(pty._psi.shape[-2] * pty._psi.shape[-1])
         else:
-            pty._psi = np.fft.fftn(pty._psi, axes=(-2, -1))
+            pty._psi = fftn(pty._psi, axes=(-2, -1))
         return pty
 
 
@@ -421,9 +425,9 @@ class IFT(CPUOperatorPtycho):
 
     def op(self, pty):
         if self.scale:
-            pty._psi = np.fft.ifftn(pty._psi, axes=(-2, -1), norm="ortho")
+            pty._psi = ifftn(pty._psi, axes=(-2, -1)) * np.sqrt(pty._psi.shape[-2] * pty._psi.shape[-1])
         else:
-            pty._psi = np.fft.ifftn(pty._psi, axes=(-2, -1))
+            pty._psi = ifftn(pty._psi, axes=(-2, -1))
         return pty
 
 
@@ -630,8 +634,7 @@ class Psi2ObjProbe(CPUOperatorPtycho):
                     for i in range(n_stack):
                         dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
                         cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                        obj_new[iobj, cy:cy + ny, cx:cx + nx] += np.fft.fftshift(
-                            p._psi[iobj, iprobe, i] / q) * pr.conj()
+                        obj_new[iobj, cy:cy + ny, cx:cx + nx] += fftshift(p._psi[iobj, iprobe, i] / q) * pr.conj()
                         obj_norm[cy:cy + ny, cx:cx + nx] += abs(pr) ** 2
             reg = obj_norm.max() * 1e-2
 
@@ -647,7 +650,7 @@ class Psi2ObjProbe(CPUOperatorPtycho):
                         dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
                         cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
                         ob = p._obj[iobj, cy:cy + ny, cx:cx + nx]
-                        p._probe[iprobe] += np.fft.fftshift(p._psi[iobj, iprobe, i] / q) * ob.conj()
+                        p._probe[iprobe] += fftshift(p._psi[iobj, iprobe, i] / q) * ob.conj()
                         probe_norm += abs(ob) ** 2
             reg = probe_norm.max() * 1e-2
             p._probe = (p._probe + reg * probe_old) / np.fmax(probe_norm + reg, 1e-12)
@@ -935,7 +938,7 @@ class ML(CPUOperatorPtycho):
                         for i in range(n_stack):
                             dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
                             cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                            obj_grad[iobj, cy:cy + ny, cx:cx + nx] += np.fft.fftshift(
+                            obj_grad[iobj, cy:cy + ny, cx:cx + nx] += fftshift(
                                 p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0]) * pr.conj()
 
             if self.update_probe:
@@ -946,7 +949,7 @@ class ML(CPUOperatorPtycho):
                             dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
                             cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
                             ob = p._obj[iobj, cy:cy + ny, cx:cx + nx]
-                            probe_grad[iprobe] += np.fft.fftshift(
+                            probe_grad[iprobe] += fftshift(
                                 p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0]) * ob.conj()
             if False:
                 # TODO: debug only
@@ -1237,6 +1240,127 @@ class AnalyseProbe(CPUOperatorPtycho):
         return p
 
 
+class PlotPositions(CPUOperatorPtycho):
+    """
+    Plot the position shifts. If the positions have not changed from the original ones, nothing is plotted or saved.
+    """
+
+    def __init__(self, verbose=True, show_plot=True, save_prefix=None, fig_size=(12, 6)):
+        """
+
+        :param verbose: if True, will print some information about the modes and/or focus
+        :param show_plot: if True, show the calculated plots in new figures.
+        :param save_prefix: e.g. "path/to/RunAnalysis01" will save plots as "path/to/RunAnalysis01-probe-modes.png" and
+                            "path/to/RunAnalysis01-probe-z.png". If None, the plots are not saved.
+        :param fig_size: the figure size
+        """
+        super(PlotPositions, self).__init__()
+        self.verbose = verbose
+        self.show_plot = show_plot
+        self.save_prefix = save_prefix
+        self.fig_size = fig_size
+
+    def op(self, p: Ptycho):
+        if self.show_plot is False and self.save_prefix is None:
+            # Dumb !
+            return p
+        x, y = p.data.posx, p.data.posy
+        dx, dy = x - p.data.posx0, y - p.data.posy0
+        r0 = np.sqrt(((x.max()) - x.min()) ** 2 + ((y.max()) - y.min()) ** 2)
+
+        if np.allclose(dx / r0, 0) and np.allclose(dy / r0, 0):
+            if self.verbose:
+                print("Average shift in positions is too small, not plotting")
+            return p
+
+        s = np.log10(r0)
+        if s < -6:
+            unit_name = "nm"
+            s = 1e9
+        elif s < -3:
+            unit_name = "µm"
+            s = 1e6
+        elif s < 0:
+            unit_name = "mm"
+            s = 1e3
+        else:
+            unit_name = "m"
+            s = 1
+
+        # Estimate average distance between neighbours to scale the arrows
+        xy = np.empty((len(x), 2))
+        xy[:, 1] = x
+        xy[:, 0] = y
+        kdt = cKDTree(xy)
+        dists = kdt.query(xy, 2)[0][:, 1]
+        avg_dists = dists.mean()
+        dr = np.sqrt(dx ** 2 + dy ** 2)
+        scale = avg_dists / dr.max()
+        if self.verbose:
+            print("Average distance between points: %5f %s" % (avg_dists * s, unit_name))
+            print("Shift in positions: mean=%f %s, max=%f%s" % (dr.mean() * s, unit_name, dr.max() * s, unit_name))
+            print("Scale:", scale)
+        if scale < 1:
+            scale = 1
+        else:
+            scale = int(np.round(scale))
+
+        if self.show_plot:
+            try:
+                fig = plt.figure(203, figsize=self.fig_size)
+                plt.clf()
+            except:
+                # no GUI or $DISPLAY
+                fig = Figure(figsize=self.fig_size)
+        else:
+            fig = Figure(figsize=self.fig_size)
+        ax = fig.add_subplot(121)
+        figdy = (6 + 1) / 72 / fig.get_size_inches()[1]
+        ax.quiver(x * s, y * s, dx * s, dy * s, scale=1 / scale, angles='xy', scale_units='xy')
+
+        ax.set_xlabel(u"x(%s)" % unit_name, horizontalalignment='left')
+        ax.set_ylabel(u"y(%s)" % unit_name)
+        fig.suptitle("Position shifts")
+        ax.set_title("arrow scale: x%d, max shift=%5f%s" % (scale, dr.max() * s, unit_name), fontsize=9)
+        ax.set_aspect('equal')
+        # plt.gca().xaxis.set_label_coords(1.05, -.05)
+        x1, x2 = ax.get_xlim()
+        y1, y2 = ax.get_ylim()
+
+        ax = fig.add_subplot(122)
+        # Create a heatmap of displacements
+        # plt.quiver(x * s, y * s, dx * s, dy * s, scale=1 / scale, angles='xy', scale_units='xy')
+        ax.scatter(x * s, y * s, 1, color='k')
+        x1, x2 = ax.get_xlim()
+        y1, y2 = ax.get_ylim()
+        grid_y, grid_x = np.mgrid[y1 / s:y2 / s:400j, x1 / s:x2 / s:400j]
+        grid_ph = griddata((x, y), np.arctan2(dy, dx), (grid_x, grid_y), method='nearest')
+        grid_ampl = griddata((x, y), dr, (grid_x, grid_y), method='cubic', fill_value=0)
+        rgba = complex2rgbalin(np.ma.masked_array(grid_ampl * np.exp(1j * grid_ph), mask=np.isnan(grid_ampl)))
+        ax.imshow(rgba, origin='lower', extent=(x1, x2, y1, y2))
+        ax.set_xlim(x1, x2)
+        ax.set_ylim(y1, y2)
+        ax = fig.add_axes((0.90, 0.20, 0.08, .08), facecolor='w')  # [left, bottom, width, height]
+        colorwheel(ax=ax)
+
+        fig.text(figdy / 5, figdy / 2, "PyNX v%s, %s" % (__version__, time.strftime("%Y/%m/%d %H:%M:%S")),
+                 fontsize=6, horizontalalignment='left', stretch='condensed')
+        if self.save_prefix is not None:
+            canvas = FigureCanvasAgg(fig)
+            filename = self.save_prefix + '-positions.png'
+            canvas.print_figure(filename, dpi=150)
+            print("Saving positions plot to:%s" % filename)
+        if self.show_plot:
+            try:
+                # Not supported on all backends (e.g. nbagg)
+                plt.draw()
+                plt.pause(.001)
+            except:
+                pass
+
+        return p
+
+
 class SelectStack(OperatorPtycho):
     """
     Operator to select a stack of observed frames to work on. This operator currently does nothing on a CPU, as
