@@ -10,6 +10,7 @@ import os
 import sys
 import time
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift, fftfreq
 from scipy.spatial import ConvexHull
 
 try:
@@ -21,6 +22,7 @@ import h5py
 from ..operator import Operator, OperatorException, has_attr_not_none
 from ..version import __version__
 from ..utils import phase
+from .shape import get_center_coord
 from ..utils.history import History
 
 
@@ -67,22 +69,25 @@ class PtychoData:
                 mask[padding:-padding, padding:-padding] = 0
 
         if iobs is not None:
-            self.iobs = np.fft.fftshift(iobs, axes=(-2, -1)).astype(np.float32)
+            self.iobs = fftshift(iobs, axes=(-2, -1)).astype(np.float32)
             # This should not be necessary
             self.iobs[self.iobs < 0] = 0
             self.iobs_sum = self.iobs.sum()
             if mask is not None:
-                self.mask = np.fft.fftshift(mask.astype(np.int8))
+                self.mask = fftshift(mask.astype(np.int8))
                 self.iobs[:, self.mask > 0] = -100
             self.scale = np.ones(len(self.iobs), dtype=np.float32)
-        self.detector_distance = detector_distance
-        self.pixel_size_detector = pixel_size_detector
-        self.wavelength = wavelength
+        self.detector_distance = float(detector_distance)
+        self.pixel_size_detector = float(pixel_size_detector)
+        self.wavelength = float(wavelength)
         if positions is not None:
             if len(positions) == 2:
                 self.posx, self.posy = positions
             else:
                 self.posx, self.posy, self.posz = positions
+            # Keep original positions, in case they are updated.
+            self.posx0 = self.posx.copy()
+            self.posy0 = self.posy.copy()
         self.near_field = near_field
 
     def pixel_size_object(self):
@@ -120,6 +125,8 @@ class Ptycho:
         self._obj = obj
         self._background = background
         self._obj_zero_phase_mask = None
+        # Use interpolation (currently bilinear, washes out details) ? Experimental.
+        self._interpolation = False
 
         if self._probe is not None:
             self._probe = self._probe.astype(np.complex64)
@@ -209,8 +216,8 @@ class Ptycho:
         # print("calc_scan_area: scan area = %8g pixels^2, center @(%6.1f, %6.1f), <d>=%6.2f)"%(c.volume, xc, yc, w))
         # Object
         ny, nx = self._obj.shape[-2:]
-        xx = np.fft.fftshift(np.fft.fftfreq(nx, d=1. / nx))
-        yy = np.fft.fftshift(np.fft.fftfreq(ny, d=1. / ny))[:, np.newaxis]
+        xx = fftshift(fftfreq(nx, d=1. / nx))
+        yy = fftshift(fftfreq(ny, d=1. / ny))[:, np.newaxis]
         self.scan_area_obj = np.ones((ny, nx), dtype=np.float32)
         vd = []
         for i in range(len(c.vertices)):
@@ -232,8 +239,8 @@ class Ptycho:
         self.scan_area_obj = self.scan_area_obj > 0.5
         # Probe
         ny, nx = self._probe.shape[-2:]
-        xx = np.fft.fftshift(np.fft.fftfreq(nx, d=1. / nx))
-        yy = np.fft.fftshift(np.fft.fftfreq(ny, d=1. / ny))[:, np.newaxis]
+        xx = fftshift(fftfreq(nx, d=1. / nx))
+        yy = fftshift(fftfreq(ny, d=1. / ny))[:, np.newaxis]
         self.scan_area_probe = np.ones((ny, nx), dtype=np.float32)
         vd = []
         for i in range(len(c.vertices)):
@@ -285,6 +292,22 @@ class Ptycho:
                 if self._background is not None:
                     if has_attr_not_none(self, '_cl_background'):
                         self._background = self._cl_background.get()
+                # Get back positions if they have been updated
+                px, py = self.data.pixel_size_object()
+                obj_shape = self._obj.shape[-2:]
+                probe_shape = self._probe.shape[-2:]
+                for v in self._cl_obs_v:
+                    vx = v.cl_x.get()
+                    vy = v.cl_y.get()
+                    for ii in range(v.npsi):
+                        x, y = get_center_coord(obj_shape, probe_shape, vx[ii], vy[ii], px, py)
+                        if False:
+                            x0, y0 = self.data.posx[v.i + ii], self.data.posy[v.i + ii]
+                            print("Pos #%3d:  x=%6.1f dx=%6.1f  y=%6.1f dy=%6.1f" %
+                                  (v.i + ii, x0 * 1e9, (x - x0) * 1e9, y0 * 1e9, (y - y0) * 1e9))
+                        self.data.posx[v.i + ii] = x
+                        self.data.posy[v.i + ii] = y
+
             if self._timestamp_counter == self._cu_timestamp_counter:
                 if has_attr_not_none(self, '_cu_obj'):
                     self._obj = self._cu_obj.get()
@@ -295,6 +318,21 @@ class Ptycho:
                 if self._background is not None:
                     if has_attr_not_none(self, '_cu_background'):
                         self._background = self._cu_background.get()
+                # Get back positions if they have been updated
+                px, py = self.data.pixel_size_object()
+                obj_shape = self._obj.shape[-2:]
+                probe_shape = self._probe.shape[-2:]
+                for v in self._cu_obs_v:
+                    vx = v.cu_x.get()
+                    vy = v.cu_y.get()
+                    for ii in range(v.npsi):
+                        x, y = get_center_coord(obj_shape, probe_shape, vx[ii], vy[ii], px, py)
+                        if False:
+                            x0, y0 = self.data.posx[v.i + ii], self.data.posy[v.i + ii]
+                            print("Pos #%3d:  x=%6.1f dx=%6.1f  y=%6.1f dy=%6.1f" %
+                                  (v.i + ii, x0 * 1e9, (x - x0) * 1e9, y0 * 1e9, (y - y0) * 1e9))
+                        self.data.posx[v.i + ii] = x
+                        self.data.posy[v.i + ii] = y
             self._cpu_timestamp_counter = self._timestamp_counter
 
     def get_obj(self):
@@ -612,7 +650,7 @@ class Ptycho:
             result_3.attrs['note'] = 'Incoherent background'
             result_3.attrs['interpretation'] = 'image'
             entry["background"] = h5py.SoftLink(entry_path + '/result_3')  # Unorthodox, departs from specification ?
-            result_3.create_dataset("data", data=np.fft.fftshift(self.get_background()), chunks=True, shuffle=True,
+            result_3.create_dataset("data", data=fftshift(self.get_background()), chunks=True, shuffle=True,
                                     compression="gzip")
             result_3.create_dataset("data_space", data="diffraction")
             result_3.create_dataset("data_type", data="intensity")
@@ -623,13 +661,29 @@ class Ptycho:
                     result_4 = entry.create_group("result_4")
                     result_4.attrs['NX_class'] = 'NXdata'
                     result_4.attrs['signal'] = 'data'
-                    result_4.attrs['note'] = 'FLoating intensities for each frame'
+                    result_4.attrs['note'] = 'Floating intensities for each frame'
                     result_4.attrs['interpretation'] = 'spectrum'
                     entry["floating_intensity"] = h5py.SoftLink(entry_path + '/result_4')
                     result_4.create_dataset("data", data=self.data.scale, compression="gzip")
                     result_4.create_dataset("data_space", data="diffraction")
                     result_4.create_dataset("data_type", data="scale")
 
+        # Save positions in entry_5
+        result_5 = entry.create_group("result_5")
+        result_5.attrs['NX_class'] = 'NXdata'
+        result_5.attrs['signal'] = 'data'
+        result_5.attrs['note'] = 'Probe positions for each frame, and shift with original data: x, y, dx, dy'
+        result_5.attrs['interpretation'] = 'spectrum'
+        entry["positions"] = h5py.SoftLink(entry_path + '/result_5')
+        s = np.empty((4, len(self.data.posx)), dtype=np.float32)
+        s[0] = self.data.posx
+        s[1] = self.data.posy
+        s[2] = self.data.posx - self.data.posx0
+        s[3] = self.data.posy - self.data.posy0
+        result_5.create_dataset("data", data=s, compression="gzip")
+        result_5.create_dataset("data_space", data="real")
+        # result_5.create_dataset("data_type", data="translation")
+
         instrument_1 = entry.create_group("instrument_1")
         instrument_1.attrs['NX_class'] = 'NXinstrument'
         if instrument is not None:
@@ -737,7 +791,8 @@ class Ptycho:
         self.history = History()
         self.cycle = 0
 
-    def update_history(self, mode='llk', update_obj=False, update_probe=False, verbose=False, **kwargs):
+    def update_history(self, mode='llk', update_obj=False, update_probe=False, update_backgroung=False,
+                       update_pos=False, verbose=False, **kwargs):
         """
         Update the history record.
         :param mode: either 'llk' (will record new log-likelihood and number of photons)
@@ -755,7 +810,7 @@ class Ptycho:
             if 'dt' in kwargs:
                 dt = kwargs['dt']
             if verbose:
-                s = algo_string(algo, self, update_obj, update_probe, False, False)
+                s = algo_string(algo, self, update_obj, update_probe, update_backgroung, update_pos)
                 print("%-10s #%3d LLK= %8.2f(p) %8.2f(g) %8.2f(e), nb photons=%e, dt/cycle=%5.3fs"
                       % (s, self.cycle, self.llk_poisson / self.nb_obs, self.llk_gaussian / self.nb_obs,
                          self.llk_euclidian / self.nb_obs, self.nb_photons_calc, dt))
@@ -793,13 +848,13 @@ def save_ptycho_data_cxi(file_name, iobs, pixel_size, wavelength, detector_dista
     if len(path):
         os.makedirs(path, exist_ok=True)
     if os.path.isfile(file_name) and overwrite is False:
-        print("CXI file already exists, not overwriting: ", file_name)
+        if verbose:
+            print("CXI file already exists, not overwriting: ", file_name)
         os.system('ls -la %s' % file_name)
         return
-    else:
+    elif verbose:
         print('Creating CXI file: %s' % file_name)
 
-
     f = h5py.File(file_name, "w")
     f.attrs['file_name'] = file_name
     f.attrs['file_time'] = time.strftime("%Y-%m-%dT%H:%M:%S%z", time.localtime(time.time()))
@@ -925,6 +980,8 @@ def algo_string(algo_base, p, update_object, update_probe, update_background=Fal
     :param p: the ptycho object
     :param update_object: True if updating the object
     :param update_probe: True if updating the probe
+    :param update_background: True if updating the background
+    :param update_pos: True if updating the positions
     :return: a short string for the algorithm
     """
     s = algo_base
@@ -944,4 +1001,7 @@ def algo_string(algo_base, p, update_object, update_probe, update_background=Fal
     if update_background:
         s += "/b"
 
+    if update_pos:
+        s += "/t"
+
     return s
