@@ -374,9 +374,9 @@ class PtychoRunnerID01(PtychoRunner):
                                 if self.params['data2cxi']:
                                     if self.params['data2cxi'] == 'crop':
                                         self.ws.center_crop_data()
-                                        self.ws.save_data_cxi(crop=True)
+                                        self.ws.ƒ(crop=True, verbose=True)
                                     else:
-                                        self.ws.save_data_cxi(crop=False)
+                                        self.ws.save_data_cxi(crop=False, verbose=True)
                                 else:
                                     self.ws.center_crop_data()
                                     self.ws.prepare()
