@@ -18,13 +18,14 @@ import copy
 import traceback
 from PIL import Image
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift
 from scipy.ndimage import zoom
 from scipy.ndimage.interpolation import rotate
 from scipy.signal import fftconvolve, medfilt2d
 from scipy.io import loadmat
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_agg import FigureCanvasAgg
-from matplotlib import pyplot as plt
+from pynx.utils.matplotlib import pyplot as plt
 
 try:
     from scipy.spatial import ConvexHull
@@ -85,6 +86,7 @@ generic (not beamline-specific) command-line arguments: (all keywords are case-i
           - object=1 or 0: activate or deactivate the object optimisation
           - background=1 or 0: activate or deactivate the background optimisation 
             (only works with AP)
+          - position=1 or 0: activate or deactivate position optimisation (for AP or ML, needs some prior convergence)
           - nbprobe=3: change the number of modes for the probe (can go up or down)
           - regularization=1e-4: setting the regularization parameter for the object, to penalize 
                                  local variations in ML runs and smooth the solution
@@ -308,11 +310,14 @@ class PtychoRunnerScan(object):
         self.dark = None
         self.iobs = None
         self.raw_x, self.raw_y, self.x, self.y = None, None, None, None
+        self.probe0 = None
+        self.obj0 = None
 
         # Default parameters for optimization
         self.update_object = True
         self.update_probe = False
         self.update_background = False
+        self.update_position = False
         self.floating_intensity = False
 
     def load_data(self):
@@ -895,67 +900,69 @@ class PtychoRunnerScan(object):
                 else:
                     oldpixelsize = (entry['probe/x_pixel_size'][()] + entry['probe/y_pixel_size'][()]) / 2
 
-        if oldpixelsize is not None:
-            print(1, oldpixelsize, pix_size_direct, np.isclose(oldpixelsize, pix_size_direct, 1e-3, 0))
-            # We loaded a probe and/or object, need to scale ?
-            if np.isclose(oldpixelsize, pix_size_direct, 1e-3, 0) is False:
-                scale = oldpixelsize / pix_size_direct
-                print(
-                    "RESCALING by factor %5.2f, pixel sizes: %5e -> %5e" % (scale, oldpixelsize, pix_size_direct))
-                # resize probe
-                nz = 1
-                if self.probe0.ndim == 3:
-                    nz = self.probe0.shape[0]
-                nold = self.probe0.shape[-1]
-                oldprobe = self.probe0.reshape((nz, nold, nold))
-                oldprobe = zoom(oldprobe.real, (1, scale, scale)) + 1j * zoom(oldprobe.imag, (1, scale, scale))
-                self.probe0 = np.zeros((nz, self.dsize, self.dsize), dtype=np.complex64)
-                nold = oldprobe.shape[-1]
-                if nold % 2:
-                    oldprobe = oldprobe[:, :-1, :-1]
+        if self.probe0 is not None:
+            # TODO: clean up this , separate scaling & reshaping probe ?
+            nold = self.probe0.shape[-1]
+            if oldpixelsize is not None or not np.isclose(nold, self.dsize):
+                if oldpixelsize is None:
+                    oldpixelsize = pix_size_direct
+                # We loaded a probe and/or object, need to scale ?
+                if not np.isclose(oldpixelsize, pix_size_direct, 1e-3, 0) or not np.isclose(nold, self.dsize):
+                    scale = oldpixelsize / pix_size_direct
+                    print("Probe: rescaling (x%5.3f) and reshaping (%d->%d pixels)" % (scale, nold, self.dsize))
+                    # resize probe
+                    nz = 1
+                    if self.probe0.ndim == 3:
+                        nz = self.probe0.shape[0]
+                    oldprobe = self.probe0.reshape((nz, nold, nold))
+                    oldprobe = zoom(oldprobe.real, (1, scale, scale)) + 1j * zoom(oldprobe.imag, (1, scale, scale))
+                    self.probe0 = np.zeros((nz, self.dsize, self.dsize), dtype=np.complex64)
                     nold = oldprobe.shape[-1]
-                if nold < self.dsize:
-                    self.probe0[:, self.dsize // 2 - nold // 2: self.dsize // 2 + nold // 2,
-                    self.dsize // 2 - nold // 2:self.dsize // 2 + nold // 2] = oldprobe
-                else:
-                    self.probe0 = oldprobe[:, nold // 2 - self.dsize // 2: nold // 2 + self.dsize // 2,
-                                  nold // 2 - self.dsize // 2:nold // 2 + self.dsize // 2]
-
-                if nz == 1:
-                    self.probe0 = self.probe0.reshape((self.dsize, self.dsize))
-
-                if self.params['load'] is not None:
-                    # Resize object as well
-                    oldobj = self.obj0
-                    nzo = 1
-                    if oldobj.ndim == 3:
-                        nzo = oldobj.shape[0]
-                    nyo, nxo = oldobj.shape[-2:]
-                    oldobj = oldobj.reshape((nzo, nyo, nxo))
-                    self.obj0 = np.zeros((nzo, self.ny, self.nx), dtype=np.complex64)
-                    oldobj = zoom(oldobj.real, (1, scale, scale)) + 1j * zoom(oldobj.imag, (1, scale, scale))
-                    nyo, nxo = oldobj.shape[-2:]
-                    if nyo % 2 == 1:
-                        if oldobj.ndim == 2:
-                            oldobj = oldobj[:-1]
-                        else:
-                            oldobj = oldobj[:, :-1]
-                        nyo -= 1
-                    if nxo % 2 == 1:
-                        if oldobj.ndim == 2:
-                            oldobj = oldobj[:, :-1]
-                        else:
-                            oldobj = oldobj[:, :, :-1]
-                        nxo -= 1
-                    if (nyo + nxo) < (self.ny + self.nx):
-                        self.obj0[:, self.ny // 2 - nyo // 2: self.ny // 2 + nyo // 2,
-                        self.nx // 2 - nxo // 2:self.nx // 2 + nxo // 2] = oldobj
+                    if nold % 2:
+                        oldprobe = oldprobe[:, :-1, :-1]
+                        nold = oldprobe.shape[-1]
+                    if nold < self.dsize:
+                        self.probe0[:, self.dsize // 2 - nold // 2: self.dsize // 2 + nold // 2,
+                        self.dsize // 2 - nold // 2:self.dsize // 2 + nold // 2] = oldprobe
                     else:
-                        self.obj0 = oldobj[:, nyo // 2 - self.ny // 2: nyo // 2 + self.ny // 2,
-                                    nxo // 2 - self.nx // 2:nxo // 2 + self.nx // 2]
+                        self.probe0 = oldprobe[:, nold // 2 - self.dsize // 2: nold // 2 + self.dsize // 2,
+                                      nold // 2 - self.dsize // 2:nold // 2 + self.dsize // 2]
 
-                    if nzo == 1:
-                        self.obj0 = self.obj0.reshape((self.ny, self.nx))
+                    if nz == 1:
+                        self.probe0 = self.probe0.reshape((self.dsize, self.dsize))
+
+                    if self.params['load'] is not None:
+                        # Resize object as well
+                        oldobj = self.obj0
+                        nzo = 1
+                        if oldobj.ndim == 3:
+                            nzo = oldobj.shape[0]
+                        nyo, nxo = oldobj.shape[-2:]
+                        oldobj = oldobj.reshape((nzo, nyo, nxo))
+                        self.obj0 = np.zeros((nzo, self.ny, self.nx), dtype=np.complex64)
+                        oldobj = zoom(oldobj.real, (1, scale, scale)) + 1j * zoom(oldobj.imag, (1, scale, scale))
+                        nyo, nxo = oldobj.shape[-2:]
+                        if nyo % 2 == 1:
+                            if oldobj.ndim == 2:
+                                oldobj = oldobj[:-1]
+                            else:
+                                oldobj = oldobj[:, :-1]
+                            nyo -= 1
+                        if nxo % 2 == 1:
+                            if oldobj.ndim == 2:
+                                oldobj = oldobj[:, :-1]
+                            else:
+                                oldobj = oldobj[:, :, :-1]
+                            nxo -= 1
+                        if (nyo + nxo) < (self.ny + self.nx):
+                            self.obj0[:, self.ny // 2 - nyo // 2: self.ny // 2 + nyo // 2,
+                            self.nx // 2 - nxo // 2:self.nx // 2 + nxo // 2] = oldobj
+                        else:
+                            self.obj0 = oldobj[:, nyo // 2 - self.ny // 2: nyo // 2 + self.ny // 2,
+                                        nxo // 2 - self.nx // 2:nxo // 2 + self.nx // 2]
+
+                        if nzo == 1:
+                            self.obj0 = self.obj0.reshape((self.ny, self.nx))
 
     def run(self):
         """
@@ -1007,7 +1014,7 @@ class PtychoRunnerScan(object):
             if self.params['defocus'] is not None and self.params['defocus'] != 0 and self.defocus_done is False:
                 self.defocus_done = True  # Don't defocus again for multiple runs of the same scan
                 if len(self.probe0.shape) == 2:
-                    self.w = wavefront.Wavefront(d=np.fft.fftshift(self.probe0.astype(np.complex64)),
+                    self.w = wavefront.Wavefront(d=fftshift(self.probe0.astype(np.complex64)),
                                                  wavelength=self.wavelength,
                                                  pixel_size=self.data.pixel_size_object()[0])
                     self.w = PropagateNearField_Wavefront(self.params['defocus']) * self.w
@@ -1016,7 +1023,7 @@ class PtychoRunnerScan(object):
                 else:
                     # Propagate all modes
                     for i in range(len(self.probe0)):
-                        self.w = wavefront.Wavefront(d=np.fft.fftshift(self.probe0[i].astype(np.complex64)),
+                        self.w = wavefront.Wavefront(d=fftshift(self.probe0[i].astype(np.complex64)),
                                                      wavelength=self.wavelength,
                                                      pixel_size=self.data.pixel_size_object()[0])
                         self.w = PropagateNearField_Wavefront(self.params['defocus']) * self.w
@@ -1087,10 +1094,9 @@ class PtychoRunnerScan(object):
                         nbcycle = int(s[0])
                     elif len(s[1].strip()):
                         nbcycle = int(s[1])
-                    print("Updating background:", self.update_background)
                     self.p = AP(update_object=self.update_object, update_probe=self.update_probe,
                                 update_background=self.update_background,
-                                show_obj_probe=show_obj_probe,
+                                show_obj_probe=show_obj_probe, update_pos=self.update_position,
                                 calc_llk=self.params['verbose'], fig_num=100) ** nbcycle * self.p
                 elif algo.lower().find('dm') >= 0:
                     realoptim = True
@@ -1114,6 +1120,7 @@ class PtychoRunnerScan(object):
                         nbcycle = int(s[1])
                     # TODO: enable background update once it's efficient with ML
                     self.p = ML(update_object=self.update_object, update_probe=self.update_probe,
+                                update_pos=self.update_position,
                                 show_obj_probe=show_obj_probe, reg_fac_obj=self.params['regularization'],
                                 calc_llk=self.params['verbose'], fig_num=100) ** nbcycle * self.p
                 elif algo.lower().find('ortho') >= 0:
@@ -1141,25 +1148,37 @@ class PtychoRunnerScan(object):
                     self.update_probe = int(algo.lower().split('probe=')[-1])
                 elif algo.lower().find('background=') >= 0:
                     self.update_background = int(algo.lower().split('background=')[-1])
+                elif algo.lower().find('position=') >= 0:
+                    self.update_position = int(algo.lower().split('position=')[-1])
+                elif algo.lower().find('positions=') >= 0:
+                    self.update_position = int(algo.lower().split('positions=')[-1])
                 elif algo.lower().find('regularization=') >= 0:
                     self.params['regularization'] = float(algo.lower().split('regularization=')[-1])
                 elif algo.lower().find('analyze') >= 0 or algo.lower().find('analysis') >= 0:
                     probe = self.p.get_probe()
                     if self.params['saveplot']:
-                        steps = "-%02d" % (self._stepnum - 1)
+                        if self._stepnum > 0:
+                            steps = "-%02d" % (self._stepnum - 1)
+                        else:
+                            # Need when self.params['save'] is not 'all'
+                            steps = ""
                         save_prefix = self.params['saveprefix'] % (self.scan, self._run) + steps
-                    else:
-                        save_prefix = None
-                    self.p = AnalyseProbe(modes=True, focus=True, verbose=True,
-                                          save_prefix=save_prefix, show_plot=False) * self.p
-                    if self.params['saveplot']:
+                        self.p = AnalyseProbe(modes=True, focus=True, verbose=True,
+                                              save_prefix=save_prefix, show_plot=False) * self.p
+                        print("Plot positions ?")
+                        self.p = PlotPositions(verbose=True, save_prefix=save_prefix, show_plot=False) * self.p
                         if os.name is 'posix':
                             try:
                                 if probe.shape[0] > 1:
                                     sf = os.path.split(save_prefix + '-probe-modes.png')
-                                    os.system('ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-probe-modes.png')))
+                                    os.system(
+                                        'ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-probe-modes.png')))
                                 sf = os.path.split(save_prefix + '-probe-z.png')
                                 os.system('ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-probe-z.png')))
+                                sf = save_prefix + '-positions.png'
+                                if os.path.isfile(sf):
+                                    sf = os.path.split(sf)
+                                    os.system('ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-positions.png')))
                             except:
                                 pass
                 else:
@@ -1213,6 +1232,10 @@ class PtychoRunnerScan(object):
                     self.update_probe = bool(int(algo.lower().split('probe=')[-1]))
                 elif algo.lower().find('background=') >= 0:
                     self.update_background = bool(int(algo.lower().split('background=')[-1]))
+                elif algo.lower().find('position=') >= 0:
+                    self.update_position = bool(int(algo.lower().split('position=')[-1]))
+                elif algo.lower().find('positions=') >= 0:
+                    self.update_position = bool(int(algo.lower().split('positions=')[-1]))
                 elif algo.lower().find('floating_intensity=') >= 0:
                     self.floating_intensity = bool(int(algo.lower().split('floating_intensity=')[-1]))
                 elif algo.lower().find('regularization=') >= 0:
@@ -1230,20 +1253,28 @@ class PtychoRunnerScan(object):
                 elif algo.lower().find('analyze') >= 0 or algo.lower().find('analysis') >= 0:
                     probe = self.p.get_probe()
                     if self.params['saveplot']:
-                        steps = "-%02d" % (self._stepnum - 1)
+                        if self._stepnum > 0:
+                            steps = "-%02d" % (self._stepnum - 1)
+                        else:
+                            # Need when self.params['save'] is not 'all'
+                            steps = ""
                         save_prefix = self.params['saveprefix'] % (self.scan, self._run) + steps
-                    else:
-                        save_prefix = None
-                    self.p = AnalyseProbe(modes=True, focus=True, verbose=True,
-                                          save_prefix=save_prefix, show_plot=False) * self.p
-                    if self.params['saveplot']:
+                        self.p = AnalyseProbe(modes=True, focus=True, verbose=True,
+                                              save_prefix=save_prefix, show_plot=False) * self.p
+                        print("Plot positions ?")
+                        self.p = PlotPositions(verbose=True, save_prefix=save_prefix, show_plot=False) * self.p
                         if os.name is 'posix':
                             try:
                                 if probe.shape[0] > 1:
                                     sf = os.path.split(save_prefix + '-probe-modes.png')
-                                    os.system('ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-probe-modes.png')))
+                                    os.system(
+                                        'ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-probe-modes.png')))
                                 sf = os.path.split(save_prefix + '-probe-z.png')
                                 os.system('ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-probe-z.png')))
+                                sf = save_prefix + '-positions.png'
+                                if os.path.isfile(sf):
+                                    sf = os.path.split(sf)
+                                    os.system('ln -sf "%s" %s' % (sf[1], os.path.join(sf[0], 'latest-positions.png')))
                             except:
                                 pass
                 elif algo.lower().find('verbose=') >= 0:
@@ -1263,7 +1294,7 @@ class PtychoRunnerScan(object):
                     # First create basic operators
                     ap = AP(update_object=self.update_object, update_probe=self.update_probe,
                             update_background=self.update_background, floating_intensity=self.floating_intensity,
-                            show_obj_probe=show_obj_probe,
+                            show_obj_probe=show_obj_probe, update_pos=self.update_position,
                             calc_llk=self.params['verbose'], fig_num=fig_num,
                             obj_smooth_sigma=self.params['obj_smooth'], obj_inertia=self.params['obj_inertia'],
                             probe_smooth_sigma=self.params['probe_smooth'], probe_inertia=self.params['probe_inertia'])
@@ -1276,7 +1307,7 @@ class PtychoRunnerScan(object):
                             center_probe_n=self.params['center_probe_n'],
                             loop_obj_probe=self.params['dm_loop_obj_probe'])
                     ml = ML(update_object=self.update_object, update_probe=self.update_probe,
-                            floating_intensity=self.floating_intensity,
+                            floating_intensity=self.floating_intensity, update_pos=self.update_position,
                             show_obj_probe=show_obj_probe, reg_fac_obj=self.params['regularization'],
                             calc_llk=self.params['verbose'], fig_num=fig_num)
 
@@ -1353,10 +1384,10 @@ class PtychoRunnerScan(object):
 
         wavelength = 12.3984e-10 / self.params['nrj']
 
-        if self.iobs is None:
-            iobs = self.raw_data
-        else:
+        if (self.iobs is not None) and crop:
             iobs = self.iobs
+        else:
+            iobs = self.raw_data
 
         if crop:
             pixel_size = self.params['pixelsize'] * self.rebinf
@@ -1364,7 +1395,10 @@ class PtychoRunnerScan(object):
             pixel_size = self.params['pixelsize']
 
         mask = None
-        if self.raw_mask is not None:
+        if (self.mask is not None) and crop:
+            if self.mask.sum() != 0:
+                mask = self.mask
+        elif self.raw_mask is not None:
             if self.raw_mask.sum() != 0:
                 mask = self.raw_mask
 
@@ -1466,9 +1500,9 @@ class PtychoRunnerScan(object):
 
         # TODO: minimise grad phase also for secondary modes ?
         obj = phase.minimize_grad_phase(self.p.get_obj()[0], center_phase=0, global_min=False,
-                                        mask=~self.p.scan_area_obj)[0]
+                                        mask=~self.p.scan_area_obj, rebin_f=2)[0]
         probe = phase.minimize_grad_phase(self.p.get_probe()[0], center_phase=0, global_min=False,
-                                          mask=~self.p.scan_area_probe)[0]
+                                          mask=~self.p.scan_area_probe, rebin_f=2)[0]
 
         if display_plot:
             try:
@@ -1867,9 +1901,9 @@ class PtychoRunner:
                 if self.params['data2cxi']:
                     if self.params['data2cxi'] == 'crop':
                         self.ws.center_crop_data()
-                        self.ws.save_data_cxi(crop=True)
+                        self.ws.save_data_cxi(crop=True, verbose=True)
                     else:
-                        self.ws.save_data_cxi()
+                        self.ws.save_data_cxi(verbose=True)
                 else:
                     if self.params['orientation_round_robin']:
                         for xy in ['x,y', 'x,-y', '-x,y', '-x,-y', 'y,x', 'y,-x', '-y,x', '-y, -x']:
