@@ -42,21 +42,25 @@ def calc_obj_shape(posx, posy, probe_shape, margin=16, multiple=4):
     return ny, nx
 
 
-def get_view_coord(obj_shape, probe_shape, dx, dy):
+def get_view_coord(obj_shape, probe_shape, dx, dy, integer=True):
     """
-    Get integer pixel coordinates of corner for the part of the object illuminated by a probe, given the shift of the probe relative to the object
-    center. Object, probe and shift correspond to 2D coordinates.
+    Get pixel coordinates of corner for the part of the object illuminated by a probe, given the shift
+    of the probe relative to the object center. Object, probe and shift correspond to 2D coordinates.
     
     Args:
         obj_shape: the shape of the object
         probe_shape: the shape of the probe
-        shift: the shift relative to the center of the object, along each dimension: (dy, dx)
+        dx: the shift relative to the center of the object, along x (in pixel units)
+        dy: the shift relative to the center of the object, along y (in pixel units)
+        integer: if True the default), will return integer coordinates - otherwise floating point values are returned
 
     Returns: a tuple (cx, cy) of the corner coordinates of the illuminated object portion.
 
     """
-    cy = int((obj_shape[0] - probe_shape[0]) // 2 + dy)
-    cx = int((obj_shape[1] - probe_shape[1]) // 2 + dx)
+    cy = (obj_shape[0] - probe_shape[0]) // 2 + dy
+    cx = (obj_shape[1] - probe_shape[1]) // 2 + dx
+    if integer:
+        cx, cy = int(cx), int(cy)
     msg = 'Getting outside of the object (dy=%d,dx=%d)(cy=%d,cx=%d). Consider increasing the object size.' \
           % (dy, dx, cy, cx)
     if cx <= 0:
@@ -72,3 +76,21 @@ def get_view_coord(obj_shape, probe_shape, dx, dy):
         cy = obj_shape[0] - probe_shape[0]
         print(msg)
     return cx, cy
+
+
+def get_center_coord(obj_shape, probe_shape, cx, cy, px=1, py=1):
+    """
+    Compute the coordinates of the center of the frame, relative to the center of the object, 
+    given the coordinates of the corner relative to the object array origin (0, 0). These are given in pixel
+    units, or in physicial unit if the pixel size is given. 
+    :param obj_shape: the object 2D shape
+    :param probe_shape: the probe 2D shape
+    :param cx: the corner coordinate of the object along x
+    :param cy: the corner coordinate of the object along y
+    :param px: the pixel size along x
+    :param py: the pixel size along y
+    :return: a tuple (dx, dy) of x and y coordinates relative to the object center
+    """
+    nyo, nxo = obj_shape
+    ny, nx = probe_shape
+    return (cx + nx / 2 - nxo / 2) * px, (cy + ny / 2 - nyo / 2) * py
