@@ -14,8 +14,11 @@ import sys
 import warnings
 import numpy as np
 from numpy import pi
+from scipy.fftpack import fftn, ifftn, fftshift
 from scipy.ndimage import zoom
-from matplotlib import pyplot as plt
+from scipy.ndimage.interpolation import shift
+# from scipy.ndimage.fourier import fourier_shift
+from pynx.utils.matplotlib import pyplot as plt
 from ..utils.plot_utils import showCplx
 from ..utils.pattern import get_img, spiral_fermat, spiral_archimedes
 from .shape import get_view_coord
@@ -131,11 +134,19 @@ def tukeywin2D(window_size, alpha=0.5):
     return np.outer(w1, w2)
 
 
-def psi(obj, probe, dx, dy, border):
-    # TODO: subpixel shift
-    cx, cy = get_view_coord(obj.shape, probe.shape, dx, dy)
+def psi(obj, probe, x, y, border):
+    cx, cy = get_view_coord(obj.shape, probe.shape, x, y, integer=False)
     ny, nx = probe.shape
-    return probe * obj[cy:cy + ny, cx:cx + nx]
+    dx, dy = x % 1, y % 1
+    cx, cy = int(round(cx - dx)), int(round(cy - dy))
+    if np.isclose(dx, 0) and np.isclose(dy, 0):
+        return probe * obj[cy:cy + ny, cx:cx + nx]
+    else:
+        o = obj[cy:cy + ny, cx:cx + nx]
+        return probe * (shift(o.real, (-dy, -dx), mode='wrap', order=2) + shift(o.imag, (-dy, -dx), mode='wrap',
+                                                                                order=2) * 1j)
+        # fourier_shift creates some diffraction from the object discontinuity on the borders
+        # return probe * fourier_shift(o, (-dy, -dx))
 
 
 class Im(object):
@@ -384,7 +395,7 @@ class Simulation(object):
                     sys.stdout.write('%d ' % (n - index))
                     sys.stdout.flush()
             self.psi.values[index] = psi(self.obj.values, self.probe.values, sx, sy, self.obj.info['border_pix'])
-            self.wd = Wavefront(d=np.fft.fftshift(self.psi.values[index]), wavelength=wavelength,
+            self.wd = Wavefront(d=fftshift(self.psi.values[index]), wavelength=wavelength,
                                 pixel_size=pixel_size_object)
             if near_field:
                 self.wd = PropagateNearField(dz=detector_distance) * self.wd
