@@ -28,9 +28,9 @@ def test_fhkl(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False,
     fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, verbose=verbose, language=language,
                                cl_platform=cl_platform)
     # Compare to analytical formula
-    fhkl_gold = (exp(1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
-                exp(1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
+    fhkl_gold = (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
+    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
+                exp(-1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
     diff = abs(fhkl - fhkl_gold)
     if return_array:
         return diff
@@ -45,7 +45,7 @@ def test_fhklo(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False
     # Create array of 3D coordinates, 50x50x50 cells
     x = arange(0, nx, dtype=float32)
     y = arange(0, ny, dtype=float32)[:, newaxis]
-    z = arange(0, nz, dtype=float32)[:, newaxis, newaxis]
+    z = arange(0, nz, dtype=float32)[:, newaxis, newaxis]  # z negative, atoms below the surface
     occ = ones((x + y + z).shape) * 0.4
     # HKL coordinates as a 2D array
     h = random.uniform(.01, .5, nh)
@@ -57,9 +57,9 @@ def test_fhklo(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False
     fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, verbose=verbose, language=language,
                                cl_platform=cl_platform)
     # Compare to analytical formula
-    fhkl_gold = occ.mean() * (exp(1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
-                exp(1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
+    fhkl_gold = occ.mean() * (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
+    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
+                exp(-1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
     diff = abs(fhkl - fhkl_gold)
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
@@ -90,9 +90,9 @@ def test_fhklo_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=
     fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, sz_imag=sz_imag, verbose=verbose,
                                language=language, cl_platform=cl_platform)
     # Compare to analytical formula
-    fhkl_gold = occ.mean() * (exp(1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (exp(-2 * pi * (1j * l + sz_imag) * nz) - 1) / (
-                exp(-2 * pi * (1j * l + sz_imag)) - 1)
+    fhkl_gold = occ.mean() * (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
+    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (exp(2 * pi * (1j * l + sz_imag) * nz) - 1) / (
+                exp(2 * pi * (1j * l + sz_imag)) - 1)
     diff = abs(fhkl - fhkl_gold)
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
@@ -122,9 +122,9 @@ def test_fhkl_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=F
     fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, sz_imag=sz_imag, verbose=verbose, language=language,
                                cl_platform=cl_platform)
     # Compare to analytical formula
-    fhkl_gold = (exp(1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (exp(-2 * pi * (1j * l + sz_imag) * nz) - 1) / (
-                exp(-2 * pi * (1j * l + sz_imag)) - 1)
+    fhkl_gold = (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
+    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (exp(2 * pi * (1j * l + sz_imag) * nz) - 1) / (
+                exp(2 * pi * (1j * l + sz_imag)) - 1)
     diff = abs(fhkl - fhkl_gold)
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
@@ -436,24 +436,9 @@ def test_all(nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, gpu_name=N
                                       cl_platform=p.name)
                 junk = test_dwba5(d.name, show_plot=False, verbose=verbose, language='OpenCL', cl_platform=p.name)
 
+
 # TODO dynamically add all tests for all available computing units and platforms
-class TestPynx(unittest.TestCase):
-    def test_PyOpenCL(self):
-        msg="Testing for pyopencl - this can fail if you only use CUDA or CPU calculations"
-        try:
-            import pyopencl
-            import_ok=True
-        except ImportError:
-            import_ok=False
-        self.assertTrue(import_ok,msg=msg)
-    def test_PyCUDA(self):
-        msg="Testing for pycuda - this can fail if you only use OpenCL or CPU calculations"
-        try:
-            import pycuda
-            import_ok=True
-        except ImportError:
-            import_ok=False
-        self.assertTrue(import_ok,msg=msg)
+class TestScattering(unittest.TestCase):
     @unittest.skipIf(sys.version_info >= (3, 0),"scipy.weave is not compatible with python>=3")
     def test_Weave(self):
         msg="Testing for scipy.weave - only available in scipy for python<3. Only needed for CPU calculations, cython can be used instead."
@@ -496,30 +481,78 @@ class TestPynx(unittest.TestCase):
             res=test_fhkl(d, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
             self.assertLess(res.mean(),1,msg=msg)
 
+    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    def test_fhklo_cuda(self):
+        n=20
+        for p in scattering.fhkl.cl.get_platforms():
+            for d in p.get_devices():
+                msg="Testing fhklo calculations: platform=OpenCL, device=%s"%d
+                res=test_fhklo(d.name, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
+                self.assertLess(res.mean(),1,msg=msg)
+
+    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    def test_fhkl_graz_cuda(self):
+        n=20
+        for p in scattering.fhkl.cl.get_platforms():
+            for d in p.get_devices():
+                msg="Testing fhkl_graz calculations: platform=OpenCL, device=%s"%d
+                res=test_fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
+                self.assertLess(res.mean(),1,msg=msg)
+
+    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    def test_fhklo_graz_cuda(self):
+        n=20
+        for p in scattering.fhkl.cl.get_platforms():
+            for d in p.get_devices():
+                msg="Testing fhklo_graz calculations: platform=OpenCL, device=%s"%d
+                res=test_fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
+                self.assertLess(res.mean(),1,msg=msg)
+
     @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
     def test_fhkl_opencl(self):
         n=20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhkl calculations: platform=CUDA, device=%s"%d
+                #if d.type & cl.device_type.GPU is False:
+                #    continue
+                msg="Testing fhkl calculations: platform=OpenCL, device=%s"%d
                 res=test_fhkl(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
                 self.assertLess(res.mean(),1,msg=msg)
 
-"""
-if __name__ == '__main__':
-    unittest.main()
-"""
+    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    def test_fhklo_opencl(self):
+        n=20
+        for p in scattering.fhkl.cl.get_platforms():
+            for d in p.get_devices():
+                msg="Testing fhklo calculations: platform=OpenCL, device=%s"%d
+                res=test_fhklo(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
+                self.assertLess(res.mean(),1,msg=msg)
+
+    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    def test_fhkl_graz_opencl(self):
+        n=20
+        for p in scattering.fhkl.cl.get_platforms():
+            for d in p.get_devices():
+                msg="Testing fhkl_graz calculations: platform=OpenCL, device=%s"%d
+                res=test_fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
+                self.assertLess(res.mean(),1,msg=msg)
+
+    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    def test_fhklo_graz_opencl(self):
+        n=20
+        for p in scattering.fhkl.cl.get_platforms():
+            for d in p.get_devices():
+                msg="Testing fhklo_graz calculations: platform=OpenCL, device=%s"%d
+                res=test_fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
+                self.assertLess(res.mean(),1,msg=msg)
+
+
 def suite():
-    testsuite = unittest.TestSuite()
-    testsuite.addTest(TestPynx("test_PyOpenCL"))
-    testsuite.addTest(TestPynx("test_PyCUDA"))
-    testsuite.addTest(TestPynx("test_Weave"))
-    testsuite.addTest(TestPynx("test_Cython"))
-    testsuite.addTest(TestPynx("test_fhkl_cpu"))
-    testsuite.addTest(TestPynx("test_fhkl_cuda"))
-    testsuite.addTest(TestPynx("test_fhkl_opencl"))
-    return testsuite
+    load_tests = unittest.defaultTestLoader.loadTestsFromTestCase
+    test_suite = unittest.TestSuite([load_tests(TestScattering)])
+    return test_suite
+
 
 if __name__ == '__main__':
     runner = unittest.TextTestRunner()
-    runner.run(suite())
\ No newline at end of file
+    runner.run(suite())
