@@ -13,7 +13,7 @@ import sys
 import gc
 import sqlite3
 import numpy as np
-import matplotlib.pyplot as plt
+from pynx.utils.matplotlib import pyplot as plt
 from pynx.processing_unit import has_cuda, has_opencl, default_processing_unit
 
 if has_opencl:
@@ -650,7 +650,7 @@ class SpeedTest(object):
         # Start from a slightly loose support
         support = (abs(x) < 20) * (abs(y) < 20) * (abs(z) < 25)
 
-        cdi = CDI(np.zeros_like(obj0), obj=obj0, support=np.fft.fftshift(support), mask=None, wavelength=1e-10,
+        cdi = CDI(np.zeros_like(obj0), obj=obj0, support=fftshift(support), mask=None, wavelength=1e-10,
                   pixel_size_detector=55e-6)
 
         cdi = ops.Calc2Obs() * cdi
