@@ -9,7 +9,7 @@
 #         Ondrej Mandula
 
 import numpy as np
-from scipy.linalg import polar
+from scipy.linalg import svd
 
 
 def primes(n):
@@ -82,43 +82,59 @@ def smaller_primes(n, maxprime=13, required_dividers=(4,), decrease=True):
         return n
 
 
-def ortho_modes(m, method='eig', verbose=False):
+def ortho_modes(m, method='eig', verbose=False, return_matrix=False, nb_mode=None, return_weights=False):
     """
     Orthogonalize modes from a N+1 dimensional array or a list/tuple of N-dimensional arrays.
-    The decomposition is such that the total intensity (i.e. (abs(m)**2).sum()) is conserved.
-    
+    The decomposition is such that the total intensity (i.e. (abs(m)**2).sum()) is conserved, and
+    the modes are orthogonal, i.e. (mo[i]*mo[j].conj()).sum()=0 for i!=j
+
     Args:
         m: the stack of modes to orthogonalize along the first dimension.
-        method: either 'eig' to use eigenvalue decomposition (the works for any array dimension),
-            or 'polar' (default) to use polar decomposition (works for a stack of 2D arrays).
-            If the arrays are not 2D, 'eig' is used regardless.
-        verbose: if True and method=='eig', the matrix of coefficients will be printed
+        method: either 'eig' to use eigenvalue decomposition,
+            or 'svd' to use singular value decomposition.
+        verbose: if True, the matrix of coefficients will be printed
+        return_matrix: if True, return the linear combination matrix
+        nb_mode: the maximum number of modes to be returned. If None, all are returned.
+        return_weights: if True, will also return the relative weights of all the modes. This is useful if nb_mode
+                         is used, and only a partial list of modes is returned.
     Returns:
-        an array (mo) with the same shape ias given in input, but with orthogonal modes,
-        i.e. (mo[i]*mo[j].conj()).sum()=0 for i!=j
-        The modes are sorted by decreasing norm.
+        an array (mo) with the same shape as given in input, but with orthogonal modes sorted by decreasing norm.
+        Then if return_matrix is True, the matrix of linear coefficients is returned
+        Then if return_weights is True, an array with the
     """
-    if 'eig' in method or m[0].ndim != 2:
+    if 'eig' in method:
+        # Eigen analysis à la ptypy
         mm = np.array([[np.vdot(p2, p1) for p1 in m] for p2 in m])
-        e, v = np.linalg.eig(mm)
-        e = (-e).argsort()
-        modes = [sum(m[i] * v[i, j] for i in range(len(m))) for j in e]
-        if verbose:
-            print("Orthonormal decomposition coefficients (rows)")
-            print(np.array2string(abs(v[:, e].transpose()), threshold=10, precision=2, floatmode='fixed'))
-        return np.array(modes)
+        s, v = np.linalg.eig(mm)
+    else:  # if 'svd' in method:
+        # Singular value decomposition
+        mm = np.reshape(m, (m.shape[0], m.size // m.shape[0]))
+        v, s, vh = svd(mm, compute_uv=True, full_matrices=False)  # , lapack_driver='gesvd'
+    e = (-s).argsort()
+    v = v[:, e]
+    # Make sure the largest coefficient is >0 to get same results for SVD and eig..
+    for j in range(len(e)):
+        if v[abs(v[:, j]).argmax(), j].real < 0:
+            v[:, j] *= -1
+    modes = np.array([sum(m[i] * v[i, j] for i in range(len(m))) for j in range(len(m))])
+    if verbose:
+        print("Orthonormal decomposition coefficients (rows)")
+        print(np.array2string((v.transpose()), threshold=10, precision=3,
+                              floatmode='fixed', suppress_small=True))
+    if nb_mode is not None:
+        nb_mode = min(len(m), nb_mode)
     else:
-        # Use polar decomposition (only for a stack of 2D arrays)
-        nz, ny, nx = m.shape
-        mm = np.reshape(m, (nz, ny * nx))
-        # Polar decomposition: m = dot(p, u), with p Hermitian matrix and u orthonormal modes
-        u, p = polar(mm, side='left')
+        nb_mode = len(m)
 
-        mo = np.empty_like(m)
-        for i in range(nz):
-            mo[i] = np.reshape(u[i], (ny, nx)) * np.sqrt((abs(p[:, i]) ** 2).sum())
-
-        return mo[np.argsort(-(abs(mo) ** 2).sum(axis=(1, 2)))]
+    if return_weights and return_matrix:
+        w = np.array([(abs(modes[i]) ** 2).sum() for i in range(len(m))]) / (abs(modes) ** 2).sum()
+        return modes[:nb_mode], v, w
+    if return_matrix:
+        return modes[:nb_mode], v
+    if return_weights:
+        w = np.array([(abs(modes[i]) ** 2).sum() for i in range(len(m))]) / (abs(modes) ** 2).sum()
+        return modes[:nb_mode], w
+    return modes[:nb_mode]
 
 
 def full_width(x, y, ratio=0.5, outer=False):
@@ -166,7 +182,7 @@ def full_width(x, y, ratio=0.5, outer=False):
     return xright - xleft
 
 
-def llk_poisson(iobs, imodel):
+def llk_poisson(iobs, imodel, imodel_min=0.1):
     """
     Compute the Poisson log-likelihood for a calculated intensity, given observed values.
     The value computed is normalized so that its asymptotic value (for a large number of points) is equal
@@ -175,6 +191,7 @@ def llk_poisson(iobs, imodel):
     Args:
         iobs: the observed intensities
         imodel: the calculated/model intensity
+        imodel_min: the minimum accepted value for imodel (to avoid infinite llk when imodel=0 and iobs>0)
 
     Returns:
         The negative log-likelihood.
@@ -185,9 +202,11 @@ def llk_poisson(iobs, imodel):
         else:
             llk = imodel
     else:
+        imodel
         llk = np.empty(iobs.shape, dtype=np.float32)
         idx = np.where(iobs.flat > 0)
-        llk.flat[idx] = np.take((imodel - iobs + iobs * np.log(iobs / imodel)).flat, idx)
+        llk.flat[idx] = np.take(
+            (imodel - iobs + iobs * np.log(iobs / (imodel + imodel_min * (imodel < imodel_min)))).flat, idx)
         idx = np.where(iobs.flat == 0)
         llk.flat[idx] = np.take(imodel.flat, idx)
         # Negative iobs are masked
