@@ -10,7 +10,7 @@
 
 import numpy as np
 from numpy import pi
-from matplotlib import pyplot as plt
+from pynx.utils.matplotlib import pyplot as plt
 from matplotlib import gridspec
 from matplotlib.colors import LinearSegmentedColormap, hsv_to_rgb
 from ..utils.phase import minimize_grad_phase
@@ -43,22 +43,21 @@ cm_phase = LinearSegmentedColormap('Phase', cm_phase_cdict)
 
 def phase2rgb(s):
     """
-    Crates RGB image with colour-coded phase from a complex 2D array
+    Crates RGB image with colour-coded phase from a complex array.
     
     Args:
-        s: a complex 2D numpy array
+        s: a complex numpy array
     
     Returns:
-        an RGBA numpy array
+        an RGBA numpy array, with one additional dimension added
     """
     ph = np.angle(s)
     t = pi / 3
-    nx, ny = s.shape
-    rgba = np.zeros((nx, ny, 4))
-    rgba[:, :, 0] = (ph < t) * (ph > -t) + (ph > t) * (ph < 2 * t) * (2 * t - ph) / t + (ph > -2 * t) * (ph < -t) * (
+    rgba = np.zeros(list(s.shape) + [4])
+    rgba[..., 0] = (ph < t) * (ph > -t) + (ph > t) * (ph < 2 * t) * (2 * t - ph) / t + (ph > -2 * t) * (ph < -t) * (
             ph + 2 * t) / t
-    rgba[:, :, 1] = (ph > t) + (ph < -2 * t) * (-2 * t - ph) / t + (ph > 0) * (ph < t) * ph / t
-    rgba[:, :, 2] = (ph < -t) + (ph > -t) * (ph < 0) * (-ph) / t + (ph > 2 * t) * (ph - 2 * t) / t
+    rgba[..., 1] = (ph > t) + (ph < -2 * t) * (-2 * t - ph) / t + (ph > 0) * (ph < t) * ph / t
+    rgba[..., 2] = (ph < -t) + (ph > -t) * (ph < 0) * (-ph) / t + (ph > 2 * t) * (ph - 2 * t) / t
     return rgba
 
 
@@ -82,7 +81,7 @@ def complex2rgbalog(s, amin=0.5, dlogs=2, smax=None, type='uint8'):
         sabs = smax * (sabs > smax) + sabs * (sabs <= smax)
     a = np.log10(sabs + 1e-20)
     a -= a.max() - dlogs  # display dlogs orders of magnitude
-    rgba[:, :, 3] = amin + a / dlogs * (1 - amin) * (a > 0)
+    rgba[..., 3] = amin + a / dlogs * (1 - amin) * (a > 0)
     if type == 'float':
         return rgba
     return (rgba * 255).astype(np.uint8)
@@ -94,7 +93,7 @@ def complex2rgbalin(s, gamma=1.0, smax=None, smin=None, percentile=(None, None),
     Optional exponent gamma is applied to the amplitude.
 
     Args:
-        s: the 2D complex data array
+        s: the complex data array (likely 2D, but can have higher dimensions)
         gamma: gamma parameter to change the brightness curve
         smax: maximum value (brightness = 1). If not supplied and percentile is not set,
               the maximum amplitude of the array is used.
@@ -108,7 +107,7 @@ def complex2rgbalin(s, gamma=1.0, smax=None, smin=None, percentile=(None, None),
                alpha when going through slices of one object
         type: either 'float': values are in the [0..1] range, or 'uint8' (0..255) (new default)
     Returns:
-        the RGBA array
+        the RGBA array, with the same diemensions as the input array, plus one additional R/G/B/A dimension appended.
     """
     rgba = phase2rgb(s)
     a = np.abs(s)
@@ -126,16 +125,16 @@ def complex2rgbalin(s, gamma=1.0, smax=None, smin=None, percentile=(None, None),
         a = (a - smin) * (a >= smin)
     a /= a.max()
     a = a ** gamma
-    rgba[:, :, 3] = alpha[0] + alpha[1] * a
+    rgba[..., 3] = alpha[0] + alpha[1] * a
     if type == 'float':
         return rgba
     return (rgba * 255).astype(np.uint8)
 
 
-def complex2rgblin_dark(s, gamma=1, smin=None, smax=None, percentile=None, phase_shift=0):
+def complex2rgbalin_dark(s, gamma=1, smin=None, smax=None, percentile=None, phase_shift=0):
     """
     Returns RGB image based on a HSV projection with hue=phase, saturation=1 and value =amplitude.
-    This yields a black-background image contrary to
+    This yields a black-background image contrary to complex2rgbalin
 
     Args:
         s: the 2D complex data array
@@ -151,7 +150,7 @@ def complex2rgblin_dark(s, gamma=1, smin=None, smax=None, percentile=None, phase
     Returns:
         the RGB array, with values between 0 and 1.
     """
-    h = ((np.angle(s) + np.pi)/ (2 * np.pi) + phase_shift / (2*np.pi)) % 1
+    h = ((np.angle(s) + np.pi) / (2 * np.pi) + phase_shift / (2 * np.pi)) % 1
     v = np.abs(s)
     if percentile is not None:
         if percentile[0] is not None:
@@ -167,7 +166,7 @@ def complex2rgblin_dark(s, gamma=1, smin=None, smax=None, percentile=None, phase
         v = (v - smin) * (v >= smin)
     v /= v.max()
     v = v ** gamma
-    hsv = np.dstack((h,np.ones_like(h),v))
+    hsv = np.dstack((h, np.ones_like(h), v))
     return hsv_to_rgb(hsv)
 
 
@@ -298,9 +297,10 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
         probe = probe[0]
     if minimize_obj_phase:
         if scan_area_obj is not None:
-            obja = np.angle(minimize_grad_phase(obj, center_phase=0, global_min=False, mask=~scan_area_obj)[0])
+            obja = np.angle(minimize_grad_phase(obj, center_phase=0, global_min=False, mask=~scan_area_obj,
+                                                rebin_f=2)[0])
         else:
-            obja = np.angle(minimize_grad_phase(obj, center_phase=0, global_min=False)[0])
+            obja = np.angle(minimize_grad_phase(obj, center_phase=0, global_min=False, rebin_f=2)[0])
     else:
         obja = np.angle(obj)
     probea = np.angle(probe)
