@@ -27,8 +27,9 @@ command-line arguments: (all keywords are case-insensitive)
 import os
 import sys
 import copy
-import matplotlib.pyplot as plt
+from pynx.utils.matplotlib import pyplot as plt
 import numpy as np
+from scipy.fftpack import fftn, ifftn, fftshift
 
 try:
     import hdf5plugin
@@ -174,8 +175,8 @@ class FSCRunner:
             print("Analyzing resolution from the PHASE of the images")
             # Remove phase gradient
             print("Removing phase gradient from both images")
-            data1 = minimize_grad_phase(data1, mask=(mask1 < 0.5), center_phase=0, global_min=False)[0]
-            data2 = minimize_grad_phase(data2, mask=(mask2 < 0.5), center_phase=0, global_min=False)[0]
+            data1 = minimize_grad_phase(data1, mask=(mask1 < 0.5), center_phase=0, global_min=False, rebin_f=2)[0]
+            data2 = minimize_grad_phase(data2, mask=(mask2 < 0.5), center_phase=0, global_min=False, rebin_f=2)[0]
             image1 = unwrap_phase(np.angle(data1))
             image2 = unwrap_phase(np.angle(data2))
             cm_imshow = cm_phase
@@ -249,8 +250,8 @@ class FSCRunner:
             else:
                 shift, error, diffphase = register_translation(image1, image2)
         print("Detected pixel offset [y,x]: [%g, %g]" % (shift[0], shift[1]))
-        offset_image2 = np.fft.ifftn(fourier_shift(np.fft.fftn(image2), shift))
-        offset_mask2 = abs(np.fft.ifftn(fourier_shift(np.fft.fftn(mask2), shift)))
+        offset_image2 = ifftn(fourier_shift(fftn(image2), shift))
+        offset_mask2 = abs(ifftn(fourier_shift(fftn(mask2), shift)))
 
         regfsc = int(np.max(shift))
         img1 = image1[0 + regfsc:-1 - regfsc, 0 + regfsc:-1 - regfsc]
