@@ -12,7 +12,8 @@ __all__ = ['Imshow', 'ImshowRGBA', 'ImshowAbs', 'ImshowAngle', 'Rebin', 'Quadrat
 import types
 import warnings
 import numpy as np
-import matplotlib.pyplot as plt
+from scipy.fftpack import fftn, ifftn, fftshift
+from pynx.utils.matplotlib import pyplot as plt
 
 from ..utils.plot_utils import complex2rgbalin, complex2rgbalog, insertColorwheel, cm_phase
 from ..utils.array import rebin
@@ -207,9 +208,9 @@ class Imshow(CPUOperatorWavefront):
 
     def pre_imshow(self, w):
         if w.get().ndim == 2:
-            d = np.fft.fftshift(w.get())
+            d = fftshift(w.get())
         else:
-            d = np.fft.fftshift(w.get()[self.i])
+            d = fftshift(w.get()[self.i])
         plt.figure(self.fig_num)
         plt.clf()
 
@@ -412,8 +413,8 @@ class QuadraticPhase(CPUOperatorWavefront):
 
     def op(self, w):
         ny, nx = w._d.shape[-2:]
-        ix = np.fft.fftshift(np.arange(-nx // 2, nx // 2))
-        iy = np.fft.fftshift(np.arange(-ny // 2, ny // 2))
+        ix = fftshift(np.arange(-nx // 2, nx // 2))
+        iy = fftshift(np.arange(-ny // 2, ny // 2))
         vy, vx = np.meshgrid(iy, ix, indexing='ij')
         w._d *= self.scale * np.exp(1j * (vx ** 2 + vy ** 2) * self.factor)
         return w
@@ -445,8 +446,8 @@ class ThinLens(CPUOperatorWavefront):
         if dphi > np.pi:
             warnings.warn(
                 "ThinLens Operator: d(phase factor)/pixel is %5.2f>PI (@half -distance from center)! Aliasing may occur" % dphi)
-        ix = np.fft.fftshift(np.arange(-nx // 2, nx // 2))
-        iy = np.fft.fftshift(np.arange(-ny // 2, ny // 2))
+        ix = fftshift(np.arange(-nx // 2, nx // 2))
+        iy = fftshift(np.arange(-ny // 2, ny // 2))
         vy, vx = np.meshgrid(iy, ix, indexing='ij')
         w._d *= np.exp(1j * (vx ** 2 + vy ** 2) * factor)
         return w
@@ -469,8 +470,8 @@ class CircularMask(CPUOperatorWavefront):
 
     def op(self, w):
         nx, ny = np.int32(w._d.shape[-1]), np.int32(w._d.shape[-2])
-        ix = np.fft.fftshift(np.arange(-nx // 2, nx // 2)) * w.pixel_size
-        iy = np.fft.fftshift(np.arange(-ny // 2, ny // 2)) * w.pixel_size
+        ix = fftshift(np.arange(-nx // 2, nx // 2)) * w.pixel_size
+        iy = fftshift(np.arange(-ny // 2, ny // 2)) * w.pixel_size
         vy, vx = np.meshgrid(iy, ix, indexing='ij')
         if self.invert:
             w._d *= (vx ** 2 + vy ** 2) > (self.radius ** 2)
@@ -498,8 +499,8 @@ class RectangularMask(CPUOperatorWavefront):
 
     def op(self, w):
         nx, ny = np.int32(w._d.shape[-1]), np.int32(w._d.shape[-2])
-        ix = np.fft.fftshift(np.arange(-nx // 2, nx // 2)) * w.pixel_size
-        iy = np.fft.fftshift(np.arange(-ny // 2, ny // 2)) * w.pixel_size
+        ix = fftshift(np.arange(-nx // 2, nx // 2)) * w.pixel_size
+        iy = fftshift(np.arange(-ny // 2, ny // 2)) * w.pixel_size
         vy, vx = np.meshgrid(iy, ix, indexing='ij')
         if self.invert:
             w._d *= ((abs(vx) <= self.width / 2) * (abs(vy) <= self.height / 2)) == False
@@ -537,7 +538,7 @@ class FT(CPUOperatorWavefront):
     """
 
     def op(self, w):
-        w._d = np.fft.fftn(w._d, norm="ortho", axes=(-2, -1))
+        w._d = fftn(w._d, axes=(-2, -1)) / np.sqrt(w._d.shape[-2] * w._d.shape[-1])
         return w
 
 
@@ -547,7 +548,7 @@ class IFT(CPUOperatorWavefront):
     """
 
     def op(self, w):
-        w._d = np.fft.ifftn(w._d, norm="ortho", axes=(-2, -1))
+        w._d = ifftn(w._d, axes=(-2, -1)) * np.sqrt(w._d.shape[-2] * w._d.shape[-1])
         return w
 
 
@@ -828,8 +829,8 @@ class BackPropagatePaganin(CPUOperatorWavefront):
         dk = np.float32(2 * np.pi / (w._d.shape[-1] * p))
 
         ny, nx = w._d.shape[-2:]
-        kx = np.fft.fftshift(np.arange(-nx // 2, nx // 2)) * dk
-        ky = np.fft.fftshift(np.arange(-ny // 2, ny // 2)) * dk
+        kx = fftshift(np.arange(-nx // 2, nx // 2)) * dk
+        ky = fftshift(np.arange(-ny // 2, ny // 2)) * dk
         ky, kx = np.meshgrid(ky, kx, indexing='ij')
         dz = self.dz
         delta = self.delta
